"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.verifyAsset = exports.default = exports.InvalidAssetError = void 0;

var _AssetType = require("./AssetType");

var _BaseState2 = _interopRequireDefault(require("./BaseState"));

var _Serializable = require("./Serializable");

var _common = _interopRequireDefault(require("./common"));

var _crypto = _interopRequireDefault(require("./crypto"));

var _utils = _interopRequireWildcard(require("./utils"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var InvalidAssetError =
/*#__PURE__*/
function (_Error) {
  _inherits(InvalidAssetError, _Error);

  function InvalidAssetError() {
    _classCallCheck(this, InvalidAssetError);

    return _possibleConstructorReturn(this, (InvalidAssetError.__proto__ || Object.getPrototypeOf(InvalidAssetError)).apply(this, arguments));
  }

  return InvalidAssetError;
}(Error);

exports.InvalidAssetError = InvalidAssetError;

var Asset =
/*#__PURE__*/
function (_BaseState) {
  _inherits(Asset, _BaseState);

  function Asset(_ref) {
    var _this;

    var version = _ref.version,
        hash = _ref.hash,
        type = _ref.type,
        name = _ref.name,
        amount = _ref.amount,
        available = _ref.available,
        precision = _ref.precision,
        feeMode = _ref.feeMode,
        fee = _ref.fee,
        feeAddress = _ref.feeAddress,
        owner = _ref.owner,
        admin = _ref.admin,
        issuer = _ref.issuer,
        expiration = _ref.expiration,
        isFrozen = _ref.isFrozen;

    _classCallCheck(this, Asset);

    _this = _possibleConstructorReturn(this, (Asset.__proto__ || Object.getPrototypeOf(Asset)).call(this, {
      version: version
    })); // eslint-disable-next-line

    Object.defineProperty(_this, "hash", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "hashHex", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "type", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "name", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "amount", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "available", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "precision", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "feeMode", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "fee", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "feeAddress", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "owner", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "admin", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "issuer", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "expiration", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "isFrozen", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "equals", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: _utils.default.equals(Asset, function (other) {
        return _common.default.uInt256Equal(_this.hash, other.hash);
      })
    });
    Object.defineProperty(_this, "serializeWire", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: (0, _Serializable.createSerializeWire)(_this.serializeWireBase.bind(_this))
    });
    verifyAsset({
      name: name,
      type: type,
      amount: amount,
      precision: precision
    });
    _this.hash = hash;
    _this.hashHex = _common.default.uInt256ToHex(hash);
    _this.type = type;
    _this.name = name;
    _this.amount = amount;
    _this.available = available || _utils.default.ZERO;
    _this.precision = precision;
    _this.feeMode = feeMode || 0;
    _this.fee = fee || _utils.default.ZERO;
    _this.feeAddress = feeAddress || _common.default.ZERO_UINT160;
    _this.owner = owner;
    _this.admin = admin;
    _this.issuer = issuer;
    _this.expiration = expiration;
    _this.isFrozen = isFrozen || false;
    return _this;
  }

  _createClass(Asset, [{
    key: "update",
    value: function update(_ref2) {
      var available = _ref2.available,
          expiration = _ref2.expiration,
          isFrozen = _ref2.isFrozen;
      return new Asset({
        hash: this.hash,
        type: this.type,
        name: this.name,
        amount: this.amount,
        precision: this.precision,
        fee: this.fee,
        feeAddress: this.feeAddress,
        owner: this.owner,
        admin: this.admin,
        issuer: this.issuer,
        available: available == null ? this.available : available,
        expiration: expiration == null ? this.expiration : expiration,
        isFrozen: isFrozen == null ? this.isFrozen : isFrozen
      });
    }
  }, {
    key: "serializeWireBase",
    value: function serializeWireBase(writer) {
      writer.writeUInt8(this.version);
      writer.writeUInt256(this.hash);
      writer.writeUInt8(this.type);
      writer.writeVarString(this.name);
      writer.writeFixed8(this.amount);
      writer.writeFixed8(this.available);
      writer.writeUInt8(this.precision);
      writer.writeUInt8(this.feeMode);
      writer.writeFixed8(this.fee);
      writer.writeUInt160(this.feeAddress);
      writer.writeECPoint(this.owner);
      writer.writeUInt160(this.admin);
      writer.writeUInt160(this.issuer);
      writer.writeUInt32LE(this.expiration);
      writer.writeBoolean(this.isFrozen);
    }
  }, {
    key: "serializeJSON",
    // eslint-disable-next-line
    value: function serializeJSON(context) {
      var name = this.name || '';

      try {
        name = JSON.parse(name);
      } catch (error) {// eslint-disable-next-line
      }

      return {
        version: this.version,
        id: _utils.JSONHelper.writeUInt256(this.hash),
        type: (0, _AssetType.toJSONAssetType)(this.type),
        name: name,
        amount: _utils.JSONHelper.writeFixed8(this.amount),
        available: _utils.JSONHelper.writeFixed8(this.available),
        precision: this.precision,
        owner: _utils.JSONHelper.writeECPoint(this.owner),
        admin: _crypto.default.scriptHashToAddress({
          addressVersion: context.addressVersion,
          scriptHash: this.admin
        }),
        issuer: _crypto.default.scriptHashToAddress({
          addressVersion: context.addressVersion,
          scriptHash: this.issuer
        }),
        expiration: this.expiration,
        frozen: this.isFrozen
      };
    }
  }], [{
    key: "deserializeWireBase",
    value: function deserializeWireBase(_ref3) {
      var reader = _ref3.reader;
      var version = reader.readUInt8();
      var hash = reader.readUInt256();
      var type = (0, _AssetType.assertAssetType)(reader.readUInt8());
      var name = reader.readVarString();
      var amount = reader.readFixed8();
      var available = reader.readFixed8();
      var precision = reader.readUInt8();
      reader.readUInt8(); // FeeMode

      var fee = reader.readFixed8();
      var feeAddress = reader.readUInt160();
      var owner = reader.readECPoint();
      var admin = reader.readUInt160();
      var issuer = reader.readUInt160();
      var expiration = reader.readUInt32LE();
      var isFrozen = reader.readBoolean();
      return new this({
        version: version,
        hash: hash,
        type: type,
        name: name,
        amount: amount,
        available: available,
        precision: precision,
        fee: fee,
        feeAddress: feeAddress,
        owner: owner,
        admin: admin,
        issuer: issuer,
        expiration: expiration,
        isFrozen: isFrozen
      });
    }
  }, {
    key: "deserializeWire",
    value: function deserializeWire(options) {
      return this.deserializeWireBase({
        context: options.context,
        reader: new _utils.BinaryReader(options.buffer)
      });
    }
  }]);

  return Asset;
}(_BaseState2.default);

exports.default = Asset;

var verifyAsset = function verifyAsset(_ref4) {
  var name = _ref4.name,
      type = _ref4.type,
      amount = _ref4.amount,
      precision = _ref4.precision;

  if (type === _AssetType.ASSET_TYPE.CREDIT_FLAG || type === _AssetType.ASSET_TYPE.DUTY_FLAG) {
    throw new InvalidAssetError('Asset type cannot be CREDIT_FLAG or DUTY_FLAG');
  }

  var nameBuffer = Buffer.from(name, 'utf8');

  if (nameBuffer.length > 1024) {
    throw new InvalidAssetError('Name too long');
  }

  if (amount.lte(_utils.default.ZERO) && !amount.eq(_common.default.NEGATIVE_SATOSHI_FIXED8)) {
    throw new InvalidAssetError('Amount must be greater than 0');
  }

  if (type === _AssetType.ASSET_TYPE.INVOICE && !amount.eq(_common.default.NEGATIVE_SATOSHI_FIXED8)) {
    throw new InvalidAssetError('Invoice assets must have unlimited amount.');
  }

  if (precision > 8) {
    throw new InvalidAssetError('Max precision is 8.');
  }

  if (!amount.eq(_utils.default.NEGATIVE_SATOSHI) && !amount.mod(_utils.default.TEN.pow(_utils.default.EIGHT.subn(precision))).eq(_utils.default.ZERO)) {
    throw new InvalidAssetError('Invalid precision for amount.');
  }
};

exports.verifyAsset = verifyAsset;