"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.BlockTransform = exports.InvalidBlockTransformEncodingError = exports.default = void 0;

var _stream = require("stream");

var _BlockBase2 = _interopRequireDefault(require("./BlockBase"));

var _Header = _interopRequireDefault(require("./Header"));

var _ScriptContainer = require("./ScriptContainer");

var _transaction = require("./transaction");

var _errors = require("./errors");

var _common = _interopRequireDefault(require("./common"));

var _crypto = _interopRequireWildcard(require("./crypto"));

var _utils = _interopRequireWildcard(require("./utils"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return _get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } }

var Block =
/*#__PURE__*/
function (_BlockBase) {
  _inherits(Block, _BlockBase);

  function Block(_ref) {
    var _this;

    var version = _ref.version,
        previousHash = _ref.previousHash,
        merkleRoot = _ref.merkleRoot,
        timestamp = _ref.timestamp,
        index = _ref.index,
        consensusData = _ref.consensusData,
        nextConsensus = _ref.nextConsensus,
        script = _ref.script,
        hash = _ref.hash,
        transactions = _ref.transactions;

    _classCallCheck(this, Block);

    _this = _possibleConstructorReturn(this, (Block.__proto__ || Object.getPrototypeOf(Block)).call(this, {
      version: version,
      previousHash: previousHash,
      merkleRoot: merkleRoot || _crypto.MerkleTree.computeRoot(transactions.map(function (transaction) {
        return transaction.hash;
      })),
      timestamp: timestamp,
      index: index,
      consensusData: consensusData,
      nextConsensus: nextConsensus,
      script: script,
      hash: hash
    }));
    Object.defineProperty(_this, "transactions", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "__blockSize", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this, "_header", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: _utils.default.lazy(function () {
        return new _Header.default({
          version: _this.version,
          previousHash: _this.previousHash,
          merkleRoot: _this.merkleRoot,
          timestamp: _this.timestamp,
          index: _this.index,
          consensusData: _this.consensusData,
          nextConsensus: _this.nextConsensus,
          script: _this.script
        });
      })
    });
    _this.transactions = transactions;
    _this.__blockSize = _utils.default.lazy(function () {
      return _get(Block.prototype.__proto__ || Object.getPrototypeOf(Block.prototype), "size", _this) + _utils.IOHelper.sizeOfArray(_this.transactions, function (transaction) {
        return transaction.size;
      });
    });
    return _this;
  }

  _createClass(Block, [{
    key: "getNetworkFee",
    value: function () {
      var _ref2 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee(context) {
        var fees;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return Promise.all(this.transactions.map(function (transaction) {
                  return transaction.getNetworkFee(context);
                }));

              case 2:
                fees = _context.sent;
                return _context.abrupt("return", fees.reduce(function (acc, fee) {
                  return acc.add(fee);
                }, _utils.default.ZERO));

              case 4:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function getNetworkFee(_x) {
        return _ref2.apply(this, arguments);
      }

      return getNetworkFee;
    }()
  }, {
    key: "getSystemFee",
    value: function getSystemFee(context) {
      return this.transactions.reduce(function (acc, transaction) {
        return acc.add(transaction.getSystemFee(context));
      }, _utils.default.ZERO);
    }
  }, {
    key: "verify",
    value: function () {
      var _ref3 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee2(options) {
        var _options$completely, completely;

        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _options$completely = options.completely, completely = _options$completely === void 0 ? false : _options$completely;

                if (!(this.transactions.length === 0 || this.transactions[0].type !== _transaction.TRANSACTION_TYPE.MINER || this.transactions.slice(1).some(function (transaction) {
                  return transaction.type === _transaction.TRANSACTION_TYPE.MINER;
                }))) {
                  _context2.next = 3;
                  break;
                }

                throw new _errors.VerifyError('Invalid miner transaction in block.');

              case 3:
                _context2.next = 5;
                return Promise.all([this._verifyBase(options), completely ? this._verifyComplete(options) : Promise.resolve()]);

              case 5:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function verify(_x2) {
        return _ref3.apply(this, arguments);
      }

      return verify;
    }()
  }, {
    key: "_verifyBase",
    value: function () {
      var _ref5 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee3(_ref4) {
        var genesisBlock, tryGetBlock, tryGetHeader, verifyScript, existingBlock, previousHeader;
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                genesisBlock = _ref4.genesisBlock, tryGetBlock = _ref4.tryGetBlock, tryGetHeader = _ref4.tryGetHeader, verifyScript = _ref4.verifyScript;

                if (!(this.hash === genesisBlock.hash)) {
                  _context3.next = 3;
                  break;
                }

                return _context3.abrupt("return");

              case 3:
                _context3.next = 5;
                return tryGetBlock({
                  hashOrIndex: this.hash
                });

              case 5:
                existingBlock = _context3.sent;

                if (!(existingBlock != null)) {
                  _context3.next = 8;
                  break;
                }

                return _context3.abrupt("return");

              case 8:
                _context3.next = 10;
                return tryGetHeader({
                  hashOrIndex: this.previousHash
                });

              case 10:
                previousHeader = _context3.sent;

                if (!(previousHeader == null)) {
                  _context3.next = 13;
                  break;
                }

                throw new _errors.VerifyError('Previous header does not exist.');

              case 13:
                if (!(previousHeader.index + 1 !== this.index)) {
                  _context3.next = 15;
                  break;
                }

                throw new _errors.VerifyError('Previous index + 1 does not match index.');

              case 15:
                if (!(previousHeader.timestamp >= this.timestamp)) {
                  _context3.next = 17;
                  break;
                }

                throw new _errors.VerifyError('Previous timestamp is greater than block.');

              case 17:
                _context3.next = 19;
                return verifyScript({
                  scriptContainer: {
                    type: _ScriptContainer.SCRIPT_CONTAINER_TYPE.BLOCK,
                    value: this
                  },
                  hash: previousHeader.nextConsensus,
                  witness: this.script
                });

              case 19:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function _verifyBase(_x3) {
        return _ref5.apply(this, arguments);
      }

      return _verifyBase;
    }()
  }, {
    key: "_verifyComplete",
    value: function () {
      var _ref6 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee4(options) {
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return Promise.all([this._verifyConsensus(options), this._verifyTransactions(options), this._verifyNetworkFee(options)]);

              case 2:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function _verifyComplete(_x4) {
        return _ref6.apply(this, arguments);
      }

      return _verifyComplete;
    }()
  }, {
    key: "_verifyConsensus",
    value: function () {
      var _ref8 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee5(_ref7) {
        var getValidators, validators;
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                getValidators = _ref7.getValidators;
                _context5.next = 3;
                return getValidators(this.transactions);

              case 3:
                validators = _context5.sent;

                if (_common.default.uInt160Equal(this.nextConsensus, _crypto.default.getConsensusAddress(validators))) {
                  _context5.next = 6;
                  break;
                }

                throw new _errors.VerifyError('Invalid next consensus address');

              case 6:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function _verifyConsensus(_x5) {
        return _ref8.apply(this, arguments);
      }

      return _verifyConsensus;
    }()
  }, {
    key: "_verifyTransactions",
    value: function () {
      var _ref9 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee6(options) {
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _context6.next = 2;
                return Promise.all(this.transactions.map(function (transaction) {
                  return transaction.verify({
                    isSpent: options.isSpent,
                    getAsset: options.getAsset,
                    getOutput: options.getOutput,
                    calculateClaimAmount: options.calculateClaimAmount,
                    verifyScript: options.verifyScript,
                    currentHeight: options.currentHeight,
                    governingToken: options.governingToken,
                    utilityToken: options.utilityToken,
                    fees: options.fees
                  });
                }));

              case 2:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function _verifyTransactions(_x6) {
        return _ref9.apply(this, arguments);
      }

      return _verifyTransactions;
    }()
  }, {
    key: "_verifyNetworkFee",
    value: function () {
      var _ref10 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee7(options) {
        var networkFee, minerTransaction, minerTransactionNetworkFee;
        return regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                _context7.next = 2;
                return this.getNetworkFee({
                  getOutput: options.getOutput,
                  governingToken: options.governingToken,
                  utilityToken: options.utilityToken,
                  fees: options.fees
                });

              case 2:
                networkFee = _context7.sent;
                minerTransaction = this.transactions.find(function (transaction) {
                  return transaction.type === _transaction.TRANSACTION_TYPE.MINER;
                });

                if (!(minerTransaction == null)) {
                  _context7.next = 6;
                  break;
                }

                throw new _errors.VerifyError('Missing miner transaction');

              case 6:
                minerTransactionNetworkFee = minerTransaction.outputs.reduce(function (acc, output) {
                  return acc.add(output.value);
                }, _utils.default.ZERO);

                if (networkFee.eq(minerTransactionNetworkFee)) {
                  _context7.next = 9;
                  break;
                }

                throw new _errors.VerifyError('Miner output does not equal network fee.');

              case 9:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function _verifyNetworkFee(_x7) {
        return _ref10.apply(this, arguments);
      }

      return _verifyNetworkFee;
    }()
  }, {
    key: "serializeWireBase",
    value: function serializeWireBase(writer) {
      _get(Block.prototype.__proto__ || Object.getPrototypeOf(Block.prototype), "serializeWireBase", this).call(this, writer);

      writer.writeArray(this.transactions, function (transaction) {
        return transaction.serializeWireBase(writer);
      });
    }
  }, {
    key: "serializeJSON",
    // eslint-disable-next-line
    value: function () {
      var _ref11 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee8(context) {
        var blockBaseJSON;
        return regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                blockBaseJSON = _get(Block.prototype.__proto__ || Object.getPrototypeOf(Block.prototype), "serializeBlockBaseJSON", this).call(this, context);
                _context8.t0 = blockBaseJSON.version;
                _context8.t1 = blockBaseJSON.hash;
                _context8.t2 = blockBaseJSON.previousblockhash;
                _context8.t3 = blockBaseJSON.merkleroot;
                _context8.t4 = blockBaseJSON.time;
                _context8.t5 = blockBaseJSON.index;
                _context8.t6 = blockBaseJSON.nonce;
                _context8.t7 = blockBaseJSON.nextconsensus;
                _context8.t8 = blockBaseJSON.script;
                _context8.next = 12;
                return Promise.all(this.transactions.map(function (transaction) {
                  return transaction.serializeJSON(context);
                }));

              case 12:
                _context8.t9 = _context8.sent;
                _context8.t10 = blockBaseJSON.size;
                _context8.t11 = blockBaseJSON.confirmations;
                return _context8.abrupt("return", {
                  version: _context8.t0,
                  hash: _context8.t1,
                  previousblockhash: _context8.t2,
                  merkleroot: _context8.t3,
                  time: _context8.t4,
                  index: _context8.t5,
                  nonce: _context8.t6,
                  nextconsensus: _context8.t7,
                  script: _context8.t8,
                  tx: _context8.t9,
                  size: _context8.t10,
                  confirmations: _context8.t11
                });

              case 16:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      function serializeJSON(_x8) {
        return _ref11.apply(this, arguments);
      }

      return serializeJSON;
    }()
  }, {
    key: "size",
    get: function get() {
      return this.__blockSize();
    }
  }, {
    key: "header",
    get: function get() {
      return this._header();
    }
  }], [{
    key: "deserializeWireBase",
    value: function deserializeWireBase(options) {
      var reader = options.reader;

      var blockBase = _get(Block.__proto__ || Object.getPrototypeOf(Block), "deserializeBlockBaseWireBase", this).call(this, options);

      var transactions = reader.readArray(function () {
        return (0, _transaction.deserializeWireBase)(options);
      }, 0x10000);

      if (transactions.length === 0) {
        throw new _errors.InvalidFormatError();
      }

      var merkleRoot = _crypto.MerkleTree.computeRoot(transactions.map(function (transaction) {
        return transaction.hash;
      }));

      if (!_common.default.uInt256Equal(merkleRoot, blockBase.merkleRoot)) {
        throw new _errors.InvalidFormatError();
      }

      return new this({
        version: blockBase.version,
        previousHash: blockBase.previousHash,
        merkleRoot: blockBase.merkleRoot,
        timestamp: blockBase.timestamp,
        index: blockBase.index,
        consensusData: blockBase.consensusData,
        nextConsensus: blockBase.nextConsensus,
        script: blockBase.script,
        transactions: transactions
      });
    }
  }, {
    key: "deserializeWire",
    value: function deserializeWire(options) {
      return this.deserializeWireBase({
        context: options.context,
        reader: new _utils.BinaryReader(options.buffer)
      });
    }
  }]);

  return Block;
}(_BlockBase2.default);

exports.default = Block;

var InvalidBlockTransformEncodingError =
/*#__PURE__*/
function (_Error) {
  _inherits(InvalidBlockTransformEncodingError, _Error);

  function InvalidBlockTransformEncodingError() {
    _classCallCheck(this, InvalidBlockTransformEncodingError);

    return _possibleConstructorReturn(this, (InvalidBlockTransformEncodingError.__proto__ || Object.getPrototypeOf(InvalidBlockTransformEncodingError)).call(this, 'Invalid Block Transform Encoding.'));
  }

  return InvalidBlockTransformEncodingError;
}(Error);

exports.InvalidBlockTransformEncodingError = InvalidBlockTransformEncodingError;
var SIZE_OF_INT32 = 4;

var BlockTransform =
/*#__PURE__*/
function (_Transform) {
  _inherits(BlockTransform, _Transform);

  function BlockTransform(context) {
    var _this2;

    _classCallCheck(this, BlockTransform);

    _this2 = _possibleConstructorReturn(this, (BlockTransform.__proto__ || Object.getPrototypeOf(BlockTransform)).call(this, {
      readableObjectMode: true
    }));
    Object.defineProperty(_this2, "context", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(_this2, "buffer", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    _this2.context = context;
    _this2.buffer = Buffer.from([]);
    return _this2;
  }

  _createClass(BlockTransform, [{
    key: "_transform",
    value: function _transform(chunk, encoding, callback) {
      var _this3 = this;

      if (typeof chunk === 'string' || encoding !== 'buffer') {
        throw new InvalidBlockTransformEncodingError();
      }

      this.buffer = Buffer.concat([this.buffer, chunk]);

      try {
        var _processBuffer2 = this._processBuffer(new _utils.BinaryReader(this.buffer)),
            remainingBuffer = _processBuffer2.remainingBuffer,
            blocks = _processBuffer2.blocks;

        this.buffer = remainingBuffer;
        blocks.reverse().forEach(function (block) {
          return _this3.push(block);
        });
        callback(null);
      } catch (error) {
        callback(error);
      }
    }
  }, {
    key: "_processBuffer",
    value: function _processBuffer(reader) {
      if (reader.remaining < SIZE_OF_INT32) {
        return {
          remainingBuffer: reader.remainingBuffer,
          blocks: []
        };
      }

      var length = reader.clone().readInt32LE(); // TODO: Not sure why this doesn't work properly with just length...

      if (reader.remaining + SIZE_OF_INT32 < length * 2) {
        return {
          remainingBuffer: reader.remainingBuffer,
          blocks: []
        };
      }

      reader.readInt32LE();
      var block = Block.deserializeWireBase({
        context: this.context,
        reader: reader
      });

      var _processBuffer3 = this._processBuffer(reader),
          remainingBuffer = _processBuffer3.remainingBuffer,
          blocks = _processBuffer3.blocks;

      blocks.push(block);
      return {
        remainingBuffer: remainingBuffer,
        blocks: blocks
      };
    }
  }]);

  return BlockTransform;
}(_stream.Transform);

exports.BlockTransform = BlockTransform;