"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _Serializable = require("./Serializable");

var _errors = require("./errors");

var _Witness = _interopRequireDefault(require("./Witness"));

var _common = _interopRequireDefault(require("./common"));

var _crypto = _interopRequireDefault(require("./crypto"));

var _utils = _interopRequireWildcard(require("./utils"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var BlockBase =
/*#__PURE__*/
function () {
  function BlockBase(_ref) {
    var _this = this;

    var version = _ref.version,
        previousHash = _ref.previousHash,
        merkleRoot = _ref.merkleRoot,
        timestamp = _ref.timestamp,
        index = _ref.index,
        consensusData = _ref.consensusData,
        nextConsensus = _ref.nextConsensus,
        script = _ref.script,
        hash = _ref.hash;

    _classCallCheck(this, BlockBase);

    Object.defineProperty(this, "version", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "previousHash", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "merkleRoot", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "timestamp", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "index", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "consensusData", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "nextConsensus", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "script", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "_hash", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "_hashHex", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: _utils.default.lazy(function () {
        return _common.default.uInt256ToHex(_this.hash);
      })
    });
    Object.defineProperty(this, "_message", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: _utils.default.lazy(function () {
        return _this.serializeUnsigned();
      })
    });
    Object.defineProperty(this, "__thisSize", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: _utils.default.lazy(function () {
        return _utils.IOHelper.sizeOfUInt32LE + _utils.IOHelper.sizeOfUInt256 + _utils.IOHelper.sizeOfUInt256 + _utils.IOHelper.sizeOfUInt32LE + _utils.IOHelper.sizeOfUInt32LE + _utils.IOHelper.sizeOfUInt64LE + _utils.IOHelper.sizeOfUInt160 + _utils.IOHelper.sizeOfUInt8 + _this.script.size;
      })
    });
    Object.defineProperty(this, "equals", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: _utils.default.equals(this.constructor, function (other) {
        return _common.default.uInt256Equal(_this.hash, other.hash);
      })
    });
    Object.defineProperty(this, "getScriptHashesForVerifying", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: _utils.default.lazyAsync(function () {
        var _ref3 = _asyncToGenerator(
        /*#__PURE__*/
        regeneratorRuntime.mark(function _callee(_ref2) {
          var getHeader, previousHeader;
          return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  getHeader = _ref2.getHeader;

                  if (!(_this.index === 0)) {
                    _context.next = 3;
                    break;
                  }

                  return _context.abrupt("return", new Set([_common.default.uInt160ToHex(_crypto.default.toScriptHash(_this.script.verification))]));

                case 3:
                  _context.next = 5;
                  return getHeader({
                    hashOrIndex: _this.previousHash
                  });

                case 5:
                  previousHeader = _context.sent;
                  return _context.abrupt("return", new Set([_common.default.uInt160ToHex(previousHeader.nextConsensus)]));

                case 7:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, this);
        }));

        return function (_x) {
          return _ref3.apply(this, arguments);
        };
      }())
    });
    Object.defineProperty(this, "serializeUnsigned", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: (0, _Serializable.createSerializeWire)(this.serializeUnsignedBase.bind(this))
    });
    Object.defineProperty(this, "serializeWire", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: (0, _Serializable.createSerializeWire)(this.serializeWireBase.bind(this))
    });
    this.version = version || 0;
    this.previousHash = previousHash;
    this.merkleRoot = merkleRoot;
    this.timestamp = timestamp;
    this.index = index;
    this.consensusData = consensusData;
    this.nextConsensus = nextConsensus;
    this.script = script;
    var hashIn = hash;
    this._hash = hashIn == null ? _utils.default.lazy(function () {
      return _crypto.default.hash256(_this.message);
    }) : function () {
      return hashIn;
    };
  }

  _createClass(BlockBase, [{
    key: "serializeUnsignedBase",
    value: function serializeUnsignedBase(writer) {
      writer.writeUInt32LE(this.version);
      writer.writeUInt256(this.previousHash);
      writer.writeUInt256(this.merkleRoot);
      writer.writeUInt32LE(this.timestamp);
      writer.writeUInt32LE(this.index);
      writer.writeUInt64LE(this.consensusData);
      writer.writeUInt160(this.nextConsensus);
    }
  }, {
    key: "serializeWireBase",
    value: function serializeWireBase(writer) {
      this.serializeUnsignedBase(writer);
      writer.writeUInt8(1);
      this.script.serializeWireBase(writer);
    }
  }, {
    key: "serializeBlockBaseJSON",
    value: function serializeBlockBaseJSON(context) {
      return {
        version: this.version,
        hash: _utils.JSONHelper.writeUInt256(this.hash),
        size: this.size,
        previousblockhash: _utils.JSONHelper.writeUInt256(this.previousHash),
        merkleroot: _utils.JSONHelper.writeUInt256(this.merkleRoot),
        time: this.timestamp,
        index: this.index,
        nonce: _utils.JSONHelper.writeUInt64LE(this.consensusData),
        nextconsensus: _crypto.default.scriptHashToAddress({
          addressVersion: context.addressVersion,
          scriptHash: this.nextConsensus
        }),
        script: this.script.serializeJSON(context),
        // TODO: Not sure if we should bother with this...
        confirmations: 0
      };
    }
  }, {
    key: "hash",
    get: function get() {
      return this._hash();
    }
  }, {
    key: "hashHex",
    get: function get() {
      return this._hashHex();
    }
  }, {
    key: "message",
    get: function get() {
      return this._message();
    }
  }, {
    key: "size",
    get: function get() {
      return this.__thisSize();
    }
  }], [{
    key: "deserializeBlockBaseWireBase",
    value: function deserializeBlockBaseWireBase(options) {
      var reader = options.reader;
      var version = reader.readUInt32LE();
      var previousHash = reader.readUInt256();
      var merkleRoot = reader.readUInt256();
      var timestamp = reader.readUInt32LE();
      var index = reader.readUInt32LE();
      var consensusData = reader.readUInt64LE();
      var nextConsensus = reader.readUInt160();

      if (reader.readUInt8() !== 1) {
        throw new _errors.InvalidFormatError();
      }

      var script = _Witness.default.deserializeWireBase(options);

      return {
        version: version,
        previousHash: previousHash,
        merkleRoot: merkleRoot,
        timestamp: timestamp,
        index: index,
        consensusData: consensusData,
        nextConsensus: nextConsensus,
        script: script
      };
    }
  }]);

  return BlockBase;
}();

exports.default = BlockBase;