"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deserializeWire = exports.deserializeWireBase = void 0;

var _ContractParameterType = require("./ContractParameterType");

var _Serializable = require("../Serializable");

var _SignatureContractParameter = _interopRequireDefault(require("./SignatureContractParameter"));

var _BooleanContractParameter = _interopRequireDefault(require("./BooleanContractParameter"));

var _IntegerContractParameter = _interopRequireDefault(require("./IntegerContractParameter"));

var _Hash160ContractParameter = _interopRequireDefault(require("./Hash160ContractParameter"));

var _Hash256ContractParameter = _interopRequireDefault(require("./Hash256ContractParameter"));

var _ByteArrayContractParameter = _interopRequireDefault(require("./ByteArrayContractParameter"));

var _PublicKeyContractParameter = _interopRequireDefault(require("./PublicKeyContractParameter"));

var _StringContractParameter = _interopRequireDefault(require("./StringContractParameter"));

var _ArrayContractParameter = _interopRequireDefault(require("./ArrayContractParameter"));

var _InteropInterfaceContractParameter = _interopRequireDefault(require("./InteropInterfaceContractParameter"));

var _VoidContractParameter = _interopRequireDefault(require("./VoidContractParameter"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var deserializeWireBase = function deserializeWireBase(options) {
  var reader = options.reader;
  var type = (0, _ContractParameterType.assertContractParameterType)(reader.clone().readUInt8());

  switch (type) {
    case 0x00:
      return _SignatureContractParameter.default.deserializeWireBase(options);

    case 0x01:
      return _BooleanContractParameter.default.deserializeWireBase(options);

    case 0x02:
      return _IntegerContractParameter.default.deserializeWireBase(options);

    case 0x03:
      return _Hash160ContractParameter.default.deserializeWireBase(options);

    case 0x04:
      return _Hash256ContractParameter.default.deserializeWireBase(options);

    case 0x05:
      return _ByteArrayContractParameter.default.deserializeWireBase(options);

    case 0x06:
      return _PublicKeyContractParameter.default.deserializeWireBase(options);

    case 0x07:
      return _StringContractParameter.default.deserializeWireBase(options);

    case 0x10:
      return _ArrayContractParameter.default.deserializeWireBase(options);

    case 0xf0:
      return _InteropInterfaceContractParameter.default.deserializeWireBase(options);

    case 0xff:
      return _VoidContractParameter.default.deserializeWireBase(options);

    default:
      // eslint-disable-next-line
      type;
      throw new _ContractParameterType.InvalidContractParameterTypeError(type);
  }
};

exports.deserializeWireBase = deserializeWireBase;
var deserializeWire = (0, _Serializable.createDeserializeWire)(deserializeWireBase); // $FlowFixMe

exports.deserializeWire = deserializeWire;

_ArrayContractParameter.default.deserializeWireBase = function (options) {
  var reader = options.reader;
  reader.readUInt8();
  var value = reader.readArray(function () {
    return deserializeWireBase(options);
  });
  return new _ArrayContractParameter.default(value);
}; // $FlowFixMe


_ArrayContractParameter.default.deserializeWire = (0, _Serializable.createDeserializeWire)(_ArrayContractParameter.default.deserializeWireBase.bind(_ArrayContractParameter.default));