"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toJSONContractParameterType = exports.assertContractParameterType = exports.InvalidContractParameterTypeError = exports.CONTRACT_PARAMETER_TYPE = void 0;

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var CONTRACT_PARAMETER_TYPE = {
  SIGNATURE: 0x00,
  BOOLEAN: 0x01,
  INTEGER: 0x02,
  HASH160: 0x03,
  HASH256: 0x04,
  BYTE_ARRAY: 0x05,
  PUBLIC_KEY: 0x06,
  STRING: 0x07,
  ARRAY: 0x10,
  INTEROP_INTERFACE: 0xf0,
  VOID: 0xff
};
exports.CONTRACT_PARAMETER_TYPE = CONTRACT_PARAMETER_TYPE;

var InvalidContractParameterTypeError =
/*#__PURE__*/
function (_Error) {
  _inherits(InvalidContractParameterTypeError, _Error);

  function InvalidContractParameterTypeError(contractParameterType) {
    var _this;

    _classCallCheck(this, InvalidContractParameterTypeError);

    _this = _possibleConstructorReturn(this, (InvalidContractParameterTypeError.__proto__ || Object.getPrototypeOf(InvalidContractParameterTypeError)).call(this, "Expected contract parameter type, " + "found: ".concat(contractParameterType.toString(16))));
    Object.defineProperty(_this, "contractParameterType", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    _this.contractParameterType = contractParameterType;
    return _this;
  }

  return InvalidContractParameterTypeError;
}(Error);

exports.InvalidContractParameterTypeError = InvalidContractParameterTypeError;

var assertContractParameterType = function assertContractParameterType(value) {
  switch (value) {
    case CONTRACT_PARAMETER_TYPE.SIGNATURE:
      return CONTRACT_PARAMETER_TYPE.SIGNATURE;

    case CONTRACT_PARAMETER_TYPE.BOOLEAN:
      return CONTRACT_PARAMETER_TYPE.BOOLEAN;

    case CONTRACT_PARAMETER_TYPE.INTEGER:
      return CONTRACT_PARAMETER_TYPE.INTEGER;

    case CONTRACT_PARAMETER_TYPE.HASH160:
      return CONTRACT_PARAMETER_TYPE.HASH160;

    case CONTRACT_PARAMETER_TYPE.HASH256:
      return CONTRACT_PARAMETER_TYPE.HASH256;

    case CONTRACT_PARAMETER_TYPE.BYTE_ARRAY:
      return CONTRACT_PARAMETER_TYPE.BYTE_ARRAY;

    case CONTRACT_PARAMETER_TYPE.PUBLIC_KEY:
      return CONTRACT_PARAMETER_TYPE.PUBLIC_KEY;

    case CONTRACT_PARAMETER_TYPE.STRING:
      return CONTRACT_PARAMETER_TYPE.STRING;

    case CONTRACT_PARAMETER_TYPE.ARRAY:
      return CONTRACT_PARAMETER_TYPE.ARRAY;

    case CONTRACT_PARAMETER_TYPE.INTEROP_INTERFACE:
      return CONTRACT_PARAMETER_TYPE.INTEROP_INTERFACE;
    // TODO: Seems to be a bug in the TestNet

    case 0x16:
      return CONTRACT_PARAMETER_TYPE.ARRAY;

    case CONTRACT_PARAMETER_TYPE.VOID:
      return CONTRACT_PARAMETER_TYPE.VOID;

    default:
      throw new InvalidContractParameterTypeError(value);
  }
};

exports.assertContractParameterType = assertContractParameterType;

var toJSONContractParameterType = function toJSONContractParameterType(type) {
  switch (type) {
    case 0x00:
      return 'Signature';

    case 0x01:
      return 'Boolean';

    case 0x02:
      return 'Integer';

    case 0x03:
      return 'Hash160';

    case 0x04:
      return 'Hash256';

    case 0x05:
      return 'ByteArray';

    case 0x06:
      return 'PublicKey';

    case 0x07:
      return 'String';

    case 0x10:
      return 'Array';

    case 0xf0:
      return 'InteropInterface';

    case 0xff:
      return 'Void';

    default:
      // eslint-disable-next-line
      type;
      throw new InvalidContractParameterTypeError(type);
  }
};

exports.toJSONContractParameterType = toJSONContractParameterType;