"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _common = _interopRequireDefault(require("../common"));

var _crypto = _interopRequireDefault(require("./crypto"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var InvalidMerkleTreeException =
/*#__PURE__*/
function (_Error) {
  _inherits(InvalidMerkleTreeException, _Error);

  function InvalidMerkleTreeException() {
    _classCallCheck(this, InvalidMerkleTreeException);

    return _possibleConstructorReturn(this, (InvalidMerkleTreeException.__proto__ || Object.getPrototypeOf(InvalidMerkleTreeException)).call(this, 'Invalid Merkle tree.'));
  }

  return InvalidMerkleTreeException;
}(Error);

var MerkleTreeNode =
/*#__PURE__*/
function () {
  function MerkleTreeNode(_ref) {
    var hash = _ref.hash,
        leftChild = _ref.leftChild,
        rightChild = _ref.rightChild,
        parent = _ref.parent;

    _classCallCheck(this, MerkleTreeNode);

    Object.defineProperty(this, "hash", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "leftChild", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "rightChild", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "parent", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    this.hash = hash;
    this.leftChild = leftChild;
    this.rightChild = rightChild;
    this.parent = parent;
  }

  _createClass(MerkleTreeNode, [{
    key: "clone",
    value: function clone(parent) {
      var self = new MerkleTreeNode({
        hash: this.hash,
        parent: parent
      });

      if (this.leftChild != null) {
        self.leftChild = this.leftChild.clone(self);
      }

      if (this.rightChild != null) {
        self.rightChild = this.rightChild.clone(self);
      }

      return self;
    }
  }]);

  return MerkleTreeNode;
}();

var build = function build(leavesIn) {
  var leaves = leavesIn;

  if (leaves.length === 0) {
    throw new InvalidMerkleTreeException();
  }

  if (leaves.length === 1) {
    return leaves[0];
  }

  var parents = [];
  var length = Math.floor((leaves.length + 1) / 2);

  for (var i = 0; i < length; i += 1) {
    var leftChild = leaves[i * 2];
    var rightChild = void 0;

    if (i * 2 + 1 === leaves.length) {
      rightChild = leftChild;
    } else {
      rightChild = leaves[i * 2 + 1];
    }

    var node = new MerkleTreeNode({
      hash: _crypto.default.hash256(Buffer.concat([_common.default.uInt256ToBuffer(leftChild.hash), _common.default.uInt256ToBuffer(rightChild.hash)])),
      leftChild: leftChild,
      rightChild: rightChild
    });
    parents[i] = node;
    leaves[i * 2].parent = node;

    if (i * 2 + 1 !== leaves.length) {
      leaves[i * 2 + 1].parent = node;
    }
  }

  return build(parents);
};

var _trim = function _trim(node, index, depth, flags) {
  if (depth === 1 || node.leftChild == null) {
    return;
  }

  if (depth === 2) {
    if (!flags[index * 2] && !flags[index * 2 + 1]) {
      // eslint-disable-next-line
      node.leftChild = null; // eslint-disable-next-line

      node.rightChild = null;
    }
  } else {
    var leftChild = node.leftChild,
        rightChild = node.rightChild;

    if (leftChild != null && rightChild != null) {
      _trim(leftChild, index * 2, depth - 1, flags);

      _trim(rightChild, index * 2 + 1, depth - 1, flags);

      if (node.leftChild != null && node.rightChild != null && node.leftChild.leftChild == null && node.rightChild.rightChild == null) {
        // eslint-disable-next-line
        node.leftChild = null; // eslint-disable-next-line

        node.rightChild = null;
      }
    }
  }
};

var _depthFirstSearch = function _depthFirstSearch(node, hashes) {
  var leftChild = node.leftChild,
      rightChild = node.rightChild;

  if (leftChild == null || rightChild == null) {
    hashes.push(node.hash);
  } else {
    _depthFirstSearch(leftChild, hashes);

    _depthFirstSearch(rightChild, hashes);
  }
};

var _depthFirstSearch2 = function _depthFirstSearch2(node) {
  var hashes = [];

  _depthFirstSearch(node, hashes);

  return hashes;
};

var MerkleTree =
/*#__PURE__*/
function () {
  function MerkleTree(hashesOrNode) {
    _classCallCheck(this, MerkleTree);

    Object.defineProperty(this, "root", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "depth", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    this.root = Array.isArray(hashesOrNode) ? build(hashesOrNode.map(function (hash) {
      return new MerkleTreeNode({
        hash: hash
      });
    })) : hashesOrNode;
    this.depth = 1;

    for (var node = this.root; node.leftChild != null; node = node.leftChild) {
      this.depth += 1;
    }
  }

  _createClass(MerkleTree, [{
    key: "trim",
    value: function trim(flags) {
      var result = this.root.clone();

      _trim(result, 0, this.depth, flags);

      return new MerkleTree(result);
    }
  }, {
    key: "depthFirstSearch",
    value: function depthFirstSearch() {
      return _depthFirstSearch2(this.root);
    }
  }, {
    key: "toHashArray",
    value: function toHashArray() {
      return this.depthFirstSearch();
    }
  }], [{
    key: "computeRoot",
    value: function computeRoot(hashes) {
      var tree = new this(hashes);
      return tree.root.hash;
    }
  }]);

  return MerkleTree;
}();

exports.default = MerkleTree;