"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.InvalidAddressError = exports.Base58CheckError = void 0;

var _cryptoJs = _interopRequireDefault(require("crypto-js"));

var _wif = _interopRequireDefault(require("wif"));

var _bs = _interopRequireDefault(require("bs58"));

var _elliptic = require("elliptic");

var _errors = require("../errors");

var _utils = require("../utils");

var _Witness = _interopRequireDefault(require("../Witness"));

var _common = _interopRequireDefault(require("../common"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ec = new _elliptic.ec('p256');

var Base58CheckError =
/*#__PURE__*/
function (_Error) {
  _inherits(Base58CheckError, _Error);

  function Base58CheckError() {
    _classCallCheck(this, Base58CheckError);

    return _possibleConstructorReturn(this, (Base58CheckError.__proto__ || Object.getPrototypeOf(Base58CheckError)).call(this, 'Base58 Check Decode Error.'));
  }

  return Base58CheckError;
}(Error);

exports.Base58CheckError = Base58CheckError;

var InvalidAddressError =
/*#__PURE__*/
function (_Error2) {
  _inherits(InvalidAddressError, _Error2);

  function InvalidAddressError(address) {
    var _this;

    _classCallCheck(this, InvalidAddressError);

    _this = _possibleConstructorReturn(this, (InvalidAddressError.__proto__ || Object.getPrototypeOf(InvalidAddressError)).call(this, "Invalid Address: ".concat(address)));
    Object.defineProperty(_this, "address", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    _this.address = address;
    return _this;
  }

  return InvalidAddressError;
}(Error);

exports.InvalidAddressError = InvalidAddressError;

var toBuffer = function toBuffer(wordArray) {
  return Buffer.from(wordArray.toString(_cryptoJs.default.enc.Hex), 'hex');
};

var toWordArray = function toWordArray(buffer) {
  return _cryptoJs.default.enc.Hex.parse(buffer.toString('hex'));
};

var sha1 = function sha1(value) {
  return toBuffer(_cryptoJs.default.SHA1(toWordArray(value)));
};

var sha256 = function sha256(value) {
  return toBuffer(_cryptoJs.default.SHA256(toWordArray(value)));
};

var hash160 = function hash160(value) {
  return _common.default.bufferToUInt160(toBuffer(_cryptoJs.default.RIPEMD160(_cryptoJs.default.SHA256(toWordArray(value)))));
};

var hash256 = function hash256(value) {
  return _common.default.bufferToUInt256(toBuffer(_cryptoJs.default.SHA256(_cryptoJs.default.SHA256(toWordArray(value)))));
}; // TODO: This + verify should probably handle DER format signatures as well


var sign = function sign(_ref) {
  var message = _ref.message,
      privateKey = _ref.privateKey;
  var sig = ec.sign(sha256(message), _common.default.privateKeyToBuffer(privateKey));
  return Buffer.concat([sig.r.toArrayLike(Buffer, 'be', 32), sig.s.toArrayLike(Buffer, 'be', 32)]);
};

var InvalidSignatureError =
/*#__PURE__*/
function (_Error3) {
  _inherits(InvalidSignatureError, _Error3);

  function InvalidSignatureError() {
    _classCallCheck(this, InvalidSignatureError);

    return _possibleConstructorReturn(this, (InvalidSignatureError.__proto__ || Object.getPrototypeOf(InvalidSignatureError)).call(this, 'Invalid Signature'));
  }

  return InvalidSignatureError;
}(Error);

var verify = function verify(_ref2) {
  var message = _ref2.message,
      signature = _ref2.signature,
      publicKey = _ref2.publicKey;

  if (signature.length !== 64) {
    throw new InvalidSignatureError();
  }

  var r = signature.slice(0, 32);
  var s = signature.slice(32);
  return ec.verify(sha256(message), {
    r: r,
    s: s
  }, publicKey);
};

var InvalidPrivateKeyError =
/*#__PURE__*/
function (_Error4) {
  _inherits(InvalidPrivateKeyError, _Error4);

  function InvalidPrivateKeyError() {
    _classCallCheck(this, InvalidPrivateKeyError);

    return _possibleConstructorReturn(this, (InvalidPrivateKeyError.__proto__ || Object.getPrototypeOf(InvalidPrivateKeyError)).call(this, 'Invalid Private Key'));
  }

  return InvalidPrivateKeyError;
}(Error);

var toECPointFromKeyPair = function toECPointFromKeyPair(pair) {
  return _common.default.bufferToECPoint(Buffer.from(pair.getPublic(true, 'hex'), 'hex'));
};

var getPublicKey = function getPublicKey(privateKey) {
  var key = ec.keyFromPrivate(_common.default.privateKeyToBuffer(privateKey));
  key.getPublic(true, 'hex');

  var _key$validate = key.validate(),
      result = _key$validate.result;

  if (!result) {
    throw new InvalidPrivateKeyError();
  }

  return toECPointFromKeyPair(key);
};

var createKeyPair = function createKeyPair() {
  var key = ec.genKeyPair();
  return {
    privateKey: _common.default.bufferToPrivateKey(key.getPrivate().toArrayLike(Buffer, 'be')),
    publicKey: toECPointFromKeyPair(key)
  };
};

var toScriptHash = function toScriptHash(value) {
  return hash160(value);
}; // Takes various formats and converts to standard ECPoint


var toECPoint = function toECPoint(publicKey) {
  return toECPointFromKeyPair(ec.keyFromPublic(publicKey));
};

var isInfinity = function isInfinity(ecPoint) {
  return ec.keyFromPublic(ecPoint).getPublic().isInfinity();
};

var base58Checksum = function base58Checksum(buffer) {
  return _common.default.uInt256ToBuffer(hash256(buffer)).slice(0, 4);
};

var base58CheckEncode = function base58CheckEncode(buffer) {
  var checksum = base58Checksum(buffer);
  return _bs.default.encode(Buffer.concat([buffer, checksum]));
};

var base58CheckDecode = function base58CheckDecode(value) {
  var buffer = _bs.default.decode(value);

  var payload = buffer.slice(0, -4);
  var checksum = buffer.slice(-4);
  var payloadChecksum = base58Checksum(payload);

  if (!checksum.equals(payloadChecksum)) {
    throw new Base58CheckError();
  }

  return payload;
};

var scriptHashToAddress = function scriptHashToAddress(_ref3) {
  var addressVersion = _ref3.addressVersion,
      scriptHash = _ref3.scriptHash;
  var buffer = Buffer.allocUnsafe(21);
  buffer[0] = addressVersion;

  _common.default.uInt160ToBuffer(scriptHash).copy(buffer, 1);

  return base58CheckEncode(buffer);
};

var addressToScriptHash = function addressToScriptHash(_ref4) {
  var addressVersion = _ref4.addressVersion,
      address = _ref4.address;
  var decodedAddress = base58CheckDecode(address);

  if (decodedAddress.length !== 21 || decodedAddress[0] !== addressVersion) {
    throw new InvalidAddressError(address);
  }

  return _common.default.bufferToUInt160(decodedAddress.slice(1));
};

var createInvocationScript = function createInvocationScript(message, privateKey) {
  var builder = new _utils.ScriptBuilder();
  builder.emitPush(sign({
    message: message,
    privateKey: privateKey
  }));
  return builder.build();
};

var createVerificationScript = function createVerificationScript(publicKey) {
  var builder = new _utils.ScriptBuilder();
  builder.emitPushECPoint(publicKey);
  builder.emitOp('CHECKSIG');
  return builder.build();
};

var createWitness = function createWitness(message, privateKey) {
  var verification = createVerificationScript(getPublicKey(privateKey));
  var invocation = createInvocationScript(message, privateKey);
  return new _Witness.default({
    verification: verification,
    invocation: invocation
  });
};

var getVerificationScriptHash = function getVerificationScriptHash(publicKey) {
  return toScriptHash(createVerificationScript(publicKey));
};

var compareKeys = function compareKeys(a, b) {
  var aPublic = a.getPublic();
  var bPublic = b.getPublic();
  var result = aPublic.getX().cmp(bPublic.getX());

  if (result !== 0) {
    return result;
  }

  return aPublic.getY().cmp(bPublic.getY());
};

var sortKeys = function sortKeys(publicKeys) {
  return publicKeys.map(function (publicKey) {
    return ec.keyFromPublic(publicKey);
  }).sort(compareKeys).map(function (keyPair) {
    return toECPointFromKeyPair(keyPair);
  });
};

var createMultiSignatureRedeemScript = function createMultiSignatureRedeemScript(m, publicKeys) {
  if (!(m >= 1 && m <= publicKeys.length && publicKeys.length <= 1024)) {
    // TODO: Better error
    throw new Error();
  }

  var builder = new _utils.ScriptBuilder();
  builder.emitPushInt(Math.floor(m));
  var publicKeysSorted = sortKeys(publicKeys);
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = publicKeysSorted[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _ecPoint = _step.value;
      builder.emitPushECPoint(_ecPoint);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  builder.emitPushInt(publicKeysSorted.length);
  builder.emitOp('CHECKMULTISIG');
  return builder.build();
};

var getConsensusAddress = function getConsensusAddress(validators) {
  return toScriptHash(createMultiSignatureRedeemScript(validators.length - (validators.length - 1) / 3, validators));
};

var wifToPrivateKey = function wifToPrivateKey(wif, privateKeyVersion) {
  var privateKeyDecoded = base58CheckDecode(wif);

  if (privateKeyDecoded.length !== 34 || privateKeyDecoded[0] !== privateKeyVersion || privateKeyDecoded[33] !== 0x01) {
    throw new _errors.InvalidFormatError();
  }

  return _common.default.bufferToPrivateKey(privateKeyDecoded.slice(1, 33));
};

var privateKeyToWif = function privateKeyToWif(privateKey, privateKeyVersion) {
  return _wif.default.encode(privateKeyVersion, _common.default.privateKeyToBuffer(privateKey), true);
};

var _default = {
  sha1: sha1,
  sha256: sha256,
  hash160: hash160,
  hash256: hash256,
  sign: sign,
  verify: verify,
  getPublicKey: getPublicKey,
  toScriptHash: toScriptHash,
  toECPoint: toECPoint,
  isInfinity: isInfinity,
  createKeyPair: createKeyPair,
  scriptHashToAddress: scriptHashToAddress,
  addressToScriptHash: addressToScriptHash,
  createInvocationScript: createInvocationScript,
  createVerificationScript: createVerificationScript,
  createWitness: createWitness,
  getVerificationScriptHash: getVerificationScriptHash,
  createMultiSignatureRedeemScript: createMultiSignatureRedeemScript,
  getConsensusAddress: getConsensusAddress,
  privateKeyToWif: privateKeyToWif,
  wifToPrivateKey: wifToPrivateKey
};
exports.default = _default;