"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deserializeWire = exports.deserializeWireBase = void 0;

var _TransactionType = require("./TransactionType");

var _Serializable = require("../Serializable");

var _MinerTransaction = _interopRequireDefault(require("./MinerTransaction"));

var _IssueTransaction = _interopRequireDefault(require("./IssueTransaction"));

var _ClaimTransaction = _interopRequireDefault(require("./ClaimTransaction"));

var _EnrollmentTransaction = _interopRequireDefault(require("./EnrollmentTransaction"));

var _RegisterTransaction = _interopRequireDefault(require("./RegisterTransaction"));

var _ContractTransaction = _interopRequireDefault(require("./ContractTransaction"));

var _PublishTransaction = _interopRequireDefault(require("./PublishTransaction"));

var _InvocationTransaction = _interopRequireDefault(require("./InvocationTransaction"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var deserializeWireBase = function deserializeWireBase(options) {
  var reader = options.reader;
  var type = (0, _TransactionType.assertTransactionType)(reader.clone().readUInt8());

  switch (type) {
    case 0x00:
      return _MinerTransaction.default.deserializeWireBase(options);

    case 0x01:
      return _IssueTransaction.default.deserializeWireBase(options);

    case 0x02:
      return _ClaimTransaction.default.deserializeWireBase(options);

    case 0x20:
      return _EnrollmentTransaction.default.deserializeWireBase(options);

    case 0x40:
      return _RegisterTransaction.default.deserializeWireBase(options);

    case 0x80:
      return _ContractTransaction.default.deserializeWireBase(options);

    case 0xd0:
      return _PublishTransaction.default.deserializeWireBase(options);

    case 0xd1:
      return _InvocationTransaction.default.deserializeWireBase(options);

    default:
      // eslint-disable-next-line
      type;
      throw new _TransactionType.InvalidTransactionTypeError(type);
  }
};

exports.deserializeWireBase = deserializeWireBase;
var deserializeWire = (0, _Serializable.createDeserializeWire)(deserializeWireBase);
exports.deserializeWire = deserializeWire;