"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.MAX_TRANSACTION_ATTRIBUTES = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

var _attribute = require("./attribute");

var _AssetType = require("../AssetType");

var _ScriptContainer = require("../ScriptContainer");

var _TransactionType = require("./TransactionType");

var _Serializable = require("../Serializable");

var _errors = require("../errors");

var _Input = _interopRequireDefault(require("./Input"));

var _Output = _interopRequireDefault(require("./Output"));

var _Witness = _interopRequireDefault(require("../Witness"));

var _common = _interopRequireDefault(require("../common"));

var _crypto = _interopRequireDefault(require("../crypto"));

var _common2 = require("./common");

var _utils = _interopRequireWildcard(require("../utils"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _slicedToArray(arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return _sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var getUtilityValue = function getUtilityValue(_ref) {
  var outputs = _ref.outputs,
      utilityToken = _ref.utilityToken;
  return outputs.filter(function (output) {
    return _common.default.uInt256Equal(output.asset, utilityToken.hash);
  }).reduce(function (acc, output) {
    return acc.add(output.value);
  }, _utils.default.ZERO);
};

var MAX_TRANSACTION_ATTRIBUTES = 16;
exports.MAX_TRANSACTION_ATTRIBUTES = MAX_TRANSACTION_ATTRIBUTES;

var TransactionBase =
/*#__PURE__*/
function () {
  function TransactionBase(_ref2) {
    var _this = this;

    var version = _ref2.version,
        type = _ref2.type,
        attributes = _ref2.attributes,
        inputs = _ref2.inputs,
        outputs = _ref2.outputs,
        scripts = _ref2.scripts,
        hash = _ref2.hash;

    _classCallCheck(this, TransactionBase);

    _initialiseProps.call(this);

    this.version = version == null ? this.constructor.VERSION : version;
    this.type = type;
    this.attributes = attributes || [];
    this.inputs = inputs || [];
    this.outputs = outputs || [];
    this.scripts = scripts || [];
    var hashIn = hash;
    this._hash = hashIn == null ? _utils.default.lazy(function () {
      return _crypto.default.hash256(_this.message);
    }) : function () {
      return hashIn;
    };

    if (this.attributes.length > MAX_TRANSACTION_ATTRIBUTES) {
      throw new _errors.InvalidFormatError();
    }
  }

  _createClass(TransactionBase, [{
    key: "clone",
    // eslint-disable-next-line
    value: function clone(scripts) {
      throw new Error('Not Implemented');
    }
  }, {
    key: "sign",
    value: function sign(key) {
      return this.clone(this.scripts.concat([_crypto.default.createWitness(this.serializeUnsigned(), key)]));
    }
  }, {
    key: "serializeExclusiveBase",
    // eslint-disable-next-line
    value: function serializeExclusiveBase(writer) {}
  }, {
    key: "serializeUnsignedBase",
    value: function serializeUnsignedBase(writer) {
      writer.writeUInt8(this.type);
      writer.writeUInt8(this.version);
      this.serializeExclusiveBase(writer);
      writer.writeArray(this.attributes, function (attribute) {
        attribute.serializeWireBase(writer);
      });
      writer.writeArray(this.inputs, function (input) {
        input.serializeWireBase(writer);
      });
      writer.writeArray(this.outputs, function (output) {
        output.serializeWireBase(writer);
      });
    }
  }, {
    key: "serializeWireBase",
    value: function serializeWireBase(writer) {
      this.serializeUnsignedBase(writer);
      writer.writeArray(this.scripts, function (script) {
        script.serializeWireBase(writer);
      });
    }
  }, {
    key: "serializeTransactionBaseJSON",
    value: function () {
      var _ref3 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee(context) {
        var networkFee;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return this.getNetworkFee(context.feeContext);

              case 2:
                networkFee = _context.sent;
                return _context.abrupt("return", {
                  txid: _common.default.uInt256ToString(this.hashHex),
                  size: this.size,
                  version: this.version,
                  attributes: this.attributes.map(function (attribute) {
                    return attribute.serializeJSON(context);
                  }),
                  vin: this.inputs.map(function (input) {
                    return input.serializeJSON(context);
                  }),
                  vout: this.outputs.map(function (output, index) {
                    return output.serializeJSON(context, index);
                  }),
                  scripts: this.scripts.map(function (script) {
                    return script.serializeJSON(context);
                  }),
                  sys_fee: _utils.JSONHelper.writeFixed8(this.getSystemFee(context.feeContext)),
                  net_fee: _utils.JSONHelper.writeFixed8(networkFee)
                });

              case 4:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function serializeTransactionBaseJSON(_x) {
        return _ref3.apply(this, arguments);
      }

      return serializeTransactionBaseJSON;
    }() // eslint-disable-next-line

  }, {
    key: "serializeJSON",
    value: function () {
      var _ref4 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee2(context) {
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                throw new Error('Not Implemented');

              case 1:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function serializeJSON(_x2) {
        return _ref4.apply(this, arguments);
      }

      return serializeJSON;
    }()
  }, {
    key: "getNetworkFee",
    value: function () {
      var _ref5 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee3(context) {
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                return _context3.abrupt("return", this._networkFee(context));

              case 1:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function getNetworkFee(_x3) {
        return _ref5.apply(this, arguments);
      }

      return getNetworkFee;
    }()
  }, {
    key: "getSystemFee",
    value: function getSystemFee(_ref6) {
      var fees = _ref6.fees;
      return fees[this.type] || _utils.default.ZERO;
    }
  }, {
    key: "getReferences",
    value: function getReferences(options) {
      return this.__getReferences(options);
    }
  }, {
    key: "getTransactionResults",
    value: function getTransactionResults(options) {
      return this.__getTransactionResults(options);
    }
  }, {
    key: "getScriptHashesForVerifying",
    value: function () {
      var _ref7 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee4(options) {
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                return _context4.abrupt("return", this.__baseGetScriptHashesForVerifying(options));

              case 1:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function getScriptHashesForVerifying(_x4) {
        return _ref7.apply(this, arguments);
      }

      return getScriptHashesForVerifying;
    }()
  }, {
    key: "verify",
    value: function () {
      var _ref8 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee5(options) {
        var _this2 = this;

        var _options$memPool, memPool;

        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _options$memPool = options.memPool, memPool = _options$memPool === void 0 ? [] : _options$memPool;

                if (!(0, _common2.hasDuplicateInputs)(this.inputs)) {
                  _context5.next = 3;
                  break;
                }

                throw new _errors.VerifyError('Duplicate inputs');

              case 3:
                if (!memPool.some(function (tx) {
                  return !tx.equals(_this2) && tx.inputs.some(function (input) {
                    return _this2.inputs.some(function (thisInput) {
                      return input.equals(thisInput);
                    });
                  });
                })) {
                  _context5.next = 5;
                  break;
                }

                throw new _errors.VerifyError('Input already exists in mempool');

              case 5:
                if (!(this.attributes.filter(function (attribute) {
                  return attribute.usage === _attribute.ATTRIBUTE_USAGE.ECDH02 || attribute.usage === _attribute.ATTRIBUTE_USAGE.ECDH03;
                }).length > 1)) {
                  _context5.next = 7;
                  break;
                }

                throw new _errors.VerifyError('Too many ECDH attributes.');

              case 7:
                _context5.next = 9;
                return Promise.all([this._verifyDoubleSpend(options), this._verifyOutputs(options), this._verifyTransactionResults(options), this._verifyScripts(options)]);

              case 9:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function verify(_x5) {
        return _ref8.apply(this, arguments);
      }

      return verify;
    }()
  }, {
    key: "_verifyDoubleSpend",
    value: function () {
      var _ref10 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee6(_ref9) {
        var isSpent, isDoubleSpend;
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                isSpent = _ref9.isSpent;
                _context6.next = 3;
                return _utils.async.some(this.inputs, function (input) {
                  return isSpent(input);
                });

              case 3:
                isDoubleSpend = _context6.sent;

                if (!isDoubleSpend) {
                  _context6.next = 6;
                  break;
                }

                throw new _errors.VerifyError('Transaction is a double spend');

              case 6:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function _verifyDoubleSpend(_x6) {
        return _ref10.apply(this, arguments);
      }

      return _verifyDoubleSpend;
    }()
  }, {
    key: "_verifyOutputs",
    value: function () {
      var _ref12 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee8(_ref11) {
        var getAsset, currentHeight, outputsGrouped, hasInvalidOutput;
        return regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                getAsset = _ref11.getAsset, currentHeight = _ref11.currentHeight;
                outputsGrouped = _utils.default.entries(_lodash.default.groupBy(this.outputs, function (output) {
                  return _common.default.uInt256ToHex(output.asset);
                }));
                _context8.next = 4;
                return _utils.async.some(outputsGrouped, function () {
                  var _ref14 = _asyncToGenerator(
                  /*#__PURE__*/
                  regeneratorRuntime.mark(function _callee7(_ref13) {
                    var _ref15, assetHex, outputs, asset;

                    return regeneratorRuntime.wrap(function _callee7$(_context7) {
                      while (1) {
                        switch (_context7.prev = _context7.next) {
                          case 0:
                            _ref15 = _slicedToArray(_ref13, 2), assetHex = _ref15[0], outputs = _ref15[1];
                            _context7.next = 3;
                            return getAsset({
                              hash: _common.default.hexToUInt256(assetHex)
                            });

                          case 3:
                            asset = _context7.sent;

                            if (!(asset.expiration <= currentHeight + 1 && asset.type !== _AssetType.ASSET_TYPE.GOVERNING_TOKEN && asset.type !== _AssetType.ASSET_TYPE.UTILITY_TOKEN)) {
                              _context7.next = 6;
                              break;
                            }

                            return _context7.abrupt("return", true);

                          case 6:
                            return _context7.abrupt("return", outputs.some(function (output) {
                              return !output.value.mod(_utils.default.TEN.pow(_utils.default.EIGHT.subn(asset.precision))).eq(_utils.default.ZERO);
                            }));

                          case 7:
                          case "end":
                            return _context7.stop();
                        }
                      }
                    }, _callee7, this);
                  }));

                  return function (_x8) {
                    return _ref14.apply(this, arguments);
                  };
                }());

              case 4:
                hasInvalidOutput = _context8.sent;

                if (!hasInvalidOutput) {
                  _context8.next = 7;
                  break;
                }

                throw new _errors.VerifyError('Transaction has invalid output');

              case 7:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      function _verifyOutputs(_x7) {
        return _ref12.apply(this, arguments);
      }

      return _verifyOutputs;
    }()
  }, {
    key: "_verifyTransactionResults",
    value: function () {
      var _ref17 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee9(_ref16) {
        var getOutput, utilityToken, governingToken, fees, results, resultsDestroy, feeContext, systemFee, resultsIssue;
        return regeneratorRuntime.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                getOutput = _ref16.getOutput, utilityToken = _ref16.utilityToken, governingToken = _ref16.governingToken, fees = _ref16.fees;
                _context9.next = 3;
                return this.getTransactionResults({
                  getOutput: getOutput
                });

              case 3:
                results = _context9.sent;
                resultsDestroy = _utils.default.entries(results).filter( // eslint-disable-next-line
                function (_ref18) {
                  var _ref19 = _slicedToArray(_ref18, 2),
                      _ = _ref19[0],
                      value = _ref19[1];

                  return value.gt(_utils.default.ZERO);
                });

                if (!(resultsDestroy.length > 1 || resultsDestroy.length === 1 && !_common.default.uInt256Equal(_common.default.hexToUInt256(resultsDestroy[0][0]), utilityToken.hash))) {
                  _context9.next = 7;
                  break;
                }

                throw new _errors.VerifyError('Invalid destroyed output.');

              case 7:
                feeContext = {
                  getOutput: getOutput,
                  governingToken: governingToken,
                  utilityToken: utilityToken,
                  fees: fees
                };
                systemFee = this.getSystemFee(feeContext);

                if (!(systemFee.gt(_utils.default.ZERO) && (resultsDestroy.length === 0 || resultsDestroy[0][1].lt(systemFee)))) {
                  _context9.next = 11;
                  break;
                }

                throw new _errors.VerifyError('Not enough output value for system fee.');

              case 11:
                resultsIssue = _utils.default.entries(results).filter( // eslint-disable-next-line
                function (_ref20) {
                  var _ref21 = _slicedToArray(_ref20, 2),
                      _ = _ref21[0],
                      value = _ref21[1];

                  return value.lt(_utils.default.ZERO);
                });
                _context9.t0 = this.type;
                _context9.next = _context9.t0 === _TransactionType.TRANSACTION_TYPE.MINER ? 15 : _context9.t0 === _TransactionType.TRANSACTION_TYPE.CLAIM ? 15 : _context9.t0 === _TransactionType.TRANSACTION_TYPE.ISSUE ? 18 : 21;
                break;

              case 15:
                if (!resultsIssue.some(function (_ref22) {
                  var _ref23 = _slicedToArray(_ref22, 2),
                      assetHex = _ref23[0],
                      _ = _ref23[1];

                  return !_common.default.uInt256Equal(_common.default.hexToUInt256(assetHex), utilityToken.hash);
                })) {
                  _context9.next = 17;
                  break;
                }

                throw new _errors.VerifyError('Invalid miner/claim result');

              case 17:
                return _context9.abrupt("break", 24);

              case 18:
                if (!resultsIssue.some(function (_ref24) {
                  var _ref25 = _slicedToArray(_ref24, 2),
                      assetHex = _ref25[0],
                      _ = _ref25[1];

                  return _common.default.uInt256Equal(_common.default.hexToUInt256(assetHex), utilityToken.hash);
                })) {
                  _context9.next = 20;
                  break;
                }

                throw new _errors.VerifyError('Invalid issue result');

              case 20:
                return _context9.abrupt("break", 24);

              case 21:
                if (!(resultsIssue.length > 0)) {
                  _context9.next = 23;
                  break;
                }

                throw new _errors.VerifyError('Invalid results.');

              case 23:
                return _context9.abrupt("break", 24);

              case 24:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));

      function _verifyTransactionResults(_x9) {
        return _ref17.apply(this, arguments);
      }

      return _verifyTransactionResults;
    }()
  }, {
    key: "_verifyScripts",
    value: function () {
      var _ref27 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee10(_ref26) {
        var _this3 = this;

        var getAsset, getOutput, verifyScript, hashesSet, hashes;
        return regeneratorRuntime.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                getAsset = _ref26.getAsset, getOutput = _ref26.getOutput, verifyScript = _ref26.verifyScript;
                _context10.next = 3;
                return this.getScriptHashesForVerifying({
                  getAsset: getAsset,
                  getOutput: getOutput
                });

              case 3:
                hashesSet = _context10.sent;

                if (!(hashesSet.size !== this.scripts.length)) {
                  _context10.next = 6;
                  break;
                }

                throw new _errors.VerifyError('Invalid witnesses');

              case 6:
                hashes = [].concat(_toConsumableArray(hashesSet)).sort().map(function (value) {
                  return _common.default.hexToUInt160(value);
                });
                _context10.next = 9;
                return Promise.all(_lodash.default.zip(hashes, this.scripts).map(function (_ref28) {
                  var _ref29 = _slicedToArray(_ref28, 2),
                      hash = _ref29[0],
                      witness = _ref29[1];

                  return verifyScript({
                    scriptContainer: {
                      type: _ScriptContainer.SCRIPT_CONTAINER_TYPE.TRANSACTION,
                      value: _this3
                    },
                    hash: hash,
                    witness: witness
                  });
                }));

              case 9:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10, this);
      }));

      function _verifyScripts(_x10) {
        return _ref27.apply(this, arguments);
      }

      return _verifyScripts;
    }()
  }, {
    key: "hash",
    get: function get() {
      return this._hash();
    }
  }, {
    key: "hashHex",
    get: function get() {
      return this._hashHex();
    }
  }, {
    key: "size",
    get: function get() {
      return this._size();
    }
  }, {
    key: "message",
    get: function get() {
      return this._message();
    }
  }], [{
    key: "deserializeTransactionBaseStartWireBase",
    value: function deserializeTransactionBaseStartWireBase(_ref30) {
      var reader = _ref30.reader;
      var type = (0, _TransactionType.assertTransactionType)(reader.readUInt8());
      var version = reader.readUInt8();
      return {
        type: type,
        version: version
      };
    }
  }, {
    key: "deserializeTransactionBaseEndWireBase",
    value: function deserializeTransactionBaseEndWireBase(options) {
      var reader = options.reader;
      var attributes = reader.readArray(function () {
        return (0, _attribute.deserializeWireBase)(options);
      }, MAX_TRANSACTION_ATTRIBUTES);
      var inputs = reader.readArray(function () {
        return _Input.default.deserializeWireBase(options);
      });
      var outputs = reader.readArray(function () {
        return _Output.default.deserializeWireBase(options);
      }, _utils.default.USHORT_MAX_NUMBER + 1);
      var scripts = reader.readArray(function () {
        return _Witness.default.deserializeWireBase(options);
      });
      return {
        attributes: attributes,
        inputs: inputs,
        outputs: outputs,
        scripts: scripts
      };
    }
  }, {
    key: "deserializeWireBase",
    // eslint-disable-next-line
    value: function deserializeWireBase(options) {
      throw new Error('Not Implemented');
    }
  }, {
    key: "deserializeWire",
    value: function deserializeWire(options) {
      return this.deserializeWireBase({
        context: options.context,
        reader: new _utils.BinaryReader(options.buffer)
      });
    }
  }]);

  return TransactionBase;
}();

exports.default = TransactionBase;
Object.defineProperty(TransactionBase, "VERSION", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: 0
});

var _initialiseProps = function _initialiseProps() {
  var _this4 = this;

  Object.defineProperty(this, "type", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: void 0
  });
  Object.defineProperty(this, "version", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: void 0
  });
  Object.defineProperty(this, "attributes", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: void 0
  });
  Object.defineProperty(this, "inputs", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: void 0
  });
  Object.defineProperty(this, "outputs", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: void 0
  });
  Object.defineProperty(this, "scripts", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: void 0
  });
  Object.defineProperty(this, "_hash", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: void 0
  });
  Object.defineProperty(this, "_message", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: void 0
  });
  Object.defineProperty(this, "_hashHex", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: _utils.default.lazy(function () {
      return _common.default.uInt256ToHex(_this4.hash);
    })
  });
  Object.defineProperty(this, "_size", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: _utils.default.lazy(function () {
      return _utils.IOHelper.sizeOfUInt8 + _utils.IOHelper.sizeOfArray(_this4.attributes, function (attribute) {
        return attribute.size;
      }) + _utils.IOHelper.sizeOfArray(_this4.inputs, function (input) {
        return input.size;
      }) + _utils.IOHelper.sizeOfArray(_this4.outputs, function (output) {
        return output.size;
      }) + _utils.IOHelper.sizeOfArray(_this4.scripts, function (script) {
        return script.size;
      });
    })
  });
  Object.defineProperty(this, "_message", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: _utils.default.lazy(function () {
      return _this4.serializeUnsigned();
    })
  });
  Object.defineProperty(this, "equals", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: _utils.default.equals(this.constructor, function (other) {
      return _common.default.uInt256Equal(_this4.hash, other.hash);
    })
  });
  Object.defineProperty(this, "serializeWire", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: (0, _Serializable.createSerializeWire)(this.serializeWireBase.bind(this))
  });
  Object.defineProperty(this, "serializeUnsigned", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: (0, _Serializable.createSerializeWire)(this.serializeUnsignedBase.bind(this))
  });
  Object.defineProperty(this, "_networkFee", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: _utils.default.lazyAsync(function () {
      var _ref31 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee11(context) {
        var getOutput, utilityToken, outputsForInputs, inputValue, outputValue;
        return regeneratorRuntime.wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                getOutput = context.getOutput, utilityToken = context.utilityToken;
                _context11.next = 3;
                return Promise.all(_this4.inputs.map(function (input) {
                  return getOutput(input);
                }));

              case 3:
                outputsForInputs = _context11.sent;
                inputValue = getUtilityValue({
                  outputs: outputsForInputs,
                  utilityToken: utilityToken
                });
                outputValue = getUtilityValue({
                  outputs: _this4.outputs,
                  utilityToken: utilityToken
                });
                return _context11.abrupt("return", inputValue.sub(outputValue).sub(_this4.getSystemFee(context)));

              case 7:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11, this);
      }));

      return function (_x11) {
        return _ref31.apply(this, arguments);
      };
    }())
  });
  Object.defineProperty(this, "__getReferences", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: _utils.default.lazyAsync(function () {
      var _ref33 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee12(_ref32) {
        var getOutput;
        return regeneratorRuntime.wrap(function _callee12$(_context12) {
          while (1) {
            switch (_context12.prev = _context12.next) {
              case 0:
                getOutput = _ref32.getOutput;
                return _context12.abrupt("return", Promise.all(_this4.inputs.map(function (input) {
                  return getOutput(input);
                })));

              case 2:
              case "end":
                return _context12.stop();
            }
          }
        }, _callee12, this);
      }));

      return function (_x12) {
        return _ref33.apply(this, arguments);
      };
    }())
  });
  Object.defineProperty(this, "__getTransactionResults", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: _utils.default.lazyAsync(function () {
      var _ref35 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee13(_ref34) {
        var getOutput, inputOutputs, results, addOutputs;
        return regeneratorRuntime.wrap(function _callee13$(_context13) {
          while (1) {
            switch (_context13.prev = _context13.next) {
              case 0:
                getOutput = _ref34.getOutput;
                _context13.next = 3;
                return _this4.getReferences({
                  getOutput: getOutput
                });

              case 3:
                inputOutputs = _context13.sent;
                results = {};

                addOutputs = function addOutputs(outputs, negative) {
                  var _iteratorNormalCompletion = true;
                  var _didIteratorError = false;
                  var _iteratorError = undefined;

                  try {
                    for (var _iterator = outputs[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                      var _output = _step.value;

                      var _key2 = _common.default.uInt256ToHex(_output.asset);

                      if (results[_key2] == null) {
                        results[_key2] = _utils.default.ZERO;
                      }

                      results[_key2] = results[_key2].add(negative ? _output.value.neg() : _output.value);
                    }
                  } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                  } finally {
                    try {
                      if (!_iteratorNormalCompletion && _iterator.return != null) {
                        _iterator.return();
                      }
                    } finally {
                      if (_didIteratorError) {
                        throw _iteratorError;
                      }
                    }
                  }
                };

                addOutputs(inputOutputs);
                addOutputs(_this4.outputs, true);
                return _context13.abrupt("return", _lodash.default.pickBy(results, function (value) {
                  return !value.eq(_utils.default.ZERO);
                }));

              case 9:
              case "end":
                return _context13.stop();
            }
          }
        }, _callee13, this);
      }));

      return function (_x13) {
        return _ref35.apply(this, arguments);
      };
    }())
  });
  Object.defineProperty(this, "__baseGetScriptHashesForVerifying", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: _utils.default.lazyAsync(function () {
      var _ref37 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee16(_ref36) {
        var getOutput, getAsset, _ref38, _ref39, inputHashes, outputHashes, attributeHashes;

        return regeneratorRuntime.wrap(function _callee16$(_context16) {
          while (1) {
            switch (_context16.prev = _context16.next) {
              case 0:
                getOutput = _ref36.getOutput, getAsset = _ref36.getAsset;
                _context16.next = 3;
                return Promise.all([Promise.all(_this4.inputs.map(function () {
                  var _ref40 = _asyncToGenerator(
                  /*#__PURE__*/
                  regeneratorRuntime.mark(function _callee14(input) {
                    var output;
                    return regeneratorRuntime.wrap(function _callee14$(_context14) {
                      while (1) {
                        switch (_context14.prev = _context14.next) {
                          case 0:
                            _context14.next = 2;
                            return getOutput(input);

                          case 2:
                            output = _context14.sent;
                            return _context14.abrupt("return", _common.default.uInt160ToHex(output.address));

                          case 4:
                          case "end":
                            return _context14.stop();
                        }
                      }
                    }, _callee14, this);
                  }));

                  return function (_x15) {
                    return _ref40.apply(this, arguments);
                  };
                }())), Promise.all(_this4.outputs.map(function () {
                  var _ref41 = _asyncToGenerator(
                  /*#__PURE__*/
                  regeneratorRuntime.mark(function _callee15(output) {
                    var asset;
                    return regeneratorRuntime.wrap(function _callee15$(_context15) {
                      while (1) {
                        switch (_context15.prev = _context15.next) {
                          case 0:
                            _context15.next = 2;
                            return getAsset({
                              hash: output.asset
                            });

                          case 2:
                            asset = _context15.sent;

                            if (!(0, _AssetType.hasFlag)(asset.type, _AssetType.ASSET_TYPE.DUTY_FLAG)) {
                              _context15.next = 5;
                              break;
                            }

                            return _context15.abrupt("return", _common.default.uInt160ToHex(output.address));

                          case 5:
                            return _context15.abrupt("return", null);

                          case 6:
                          case "end":
                            return _context15.stop();
                        }
                      }
                    }, _callee15, this);
                  }));

                  return function (_x16) {
                    return _ref41.apply(this, arguments);
                  };
                }())).then(function (hashes) {
                  return hashes.filter(Boolean);
                })]);

              case 3:
                _ref38 = _context16.sent;
                _ref39 = _slicedToArray(_ref38, 2);
                inputHashes = _ref39[0];
                outputHashes = _ref39[1];
                attributeHashes = _this4.attributes.map(function (attribute) {
                  return attribute instanceof _attribute.UInt160Attribute && attribute.usage === _attribute.ATTRIBUTE_USAGE.SCRIPT ? _common.default.uInt160ToHex(attribute.value) : null;
                }).filter(Boolean);
                return _context16.abrupt("return", new Set([].concat(_toConsumableArray(inputHashes), _toConsumableArray(outputHashes), _toConsumableArray(attributeHashes))));

              case 9:
              case "end":
                return _context16.stop();
            }
          }
        }, _callee16, this);
      }));

      return function (_x14) {
        return _ref37.apply(this, arguments);
      };
    }())
  });
};