"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deserializeWire = exports.deserializeWireBase = void 0;

var _AttributeUsage = require("./AttributeUsage");

var _Serializable = require("../../Serializable");

var _BufferAttribute = _interopRequireDefault(require("./BufferAttribute"));

var _ECPointAttribute = _interopRequireDefault(require("./ECPointAttribute"));

var _UInt160Attribute = _interopRequireDefault(require("./UInt160Attribute"));

var _UInt256Attribute = _interopRequireDefault(require("./UInt256Attribute"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var deserializeWireBase = function deserializeWireBase(options) {
  var reader = options.reader;
  var usage = (0, _AttributeUsage.assertAttributeUsage)(reader.clone().readUInt8());

  switch (usage) {
    case 0x20:
      return _UInt160Attribute.default.deserializeWireBase(options);

    case 0x00:
    case 0x30:
    case 0xa1:
    case 0xa2:
    case 0xa3:
    case 0xa4:
    case 0xa5:
    case 0xa6:
    case 0xa7:
    case 0xa8:
    case 0xa9:
    case 0xaa:
    case 0xab:
    case 0xac:
    case 0xad:
    case 0xae:
    case 0xaf:
      return _UInt256Attribute.default.deserializeWireBase(options);

    case 0x02:
    case 0x03:
      return _ECPointAttribute.default.deserializeWireBase(options);

    case 0x81:
    case 0x90:
    case 0xf0:
    case 0xf1:
    case 0xf2:
    case 0xf3:
    case 0xf4:
    case 0xf5:
    case 0xf6:
    case 0xf7:
    case 0xf8:
    case 0xf9:
    case 0xfa:
    case 0xfb:
    case 0xfc:
    case 0xfd:
    case 0xfe:
    case 0xff:
      return _BufferAttribute.default.deserializeWireBase(options);

    default:
      // eslint-disable-next-line
      usage;
      throw new _AttributeUsage.InvalidAttributeUsageError(usage);
  }
};

exports.deserializeWireBase = deserializeWireBase;
var deserializeWire = (0, _Serializable.createDeserializeWire)(deserializeWireBase);
exports.deserializeWire = deserializeWire;