"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toJSONAttributeUsage = exports.InvalidJSONAttributeUsageError = exports.assertAttributeUsage = exports.InvalidAttributeUsageError = exports.ATTRIBUTE_USAGE = void 0;

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ATTRIBUTE_USAGE = {
  CONTRACTHASH: 0x00,
  ECDH02: 0x02,
  ECDH03: 0x03,
  SCRIPT: 0x20,
  VOTE: 0x30,
  DESCRIPTIONURL: 0x81,
  DESCRIPTION: 0x90,
  HASH1: 0xa1,
  HASH2: 0xa2,
  HASH3: 0xa3,
  HASH4: 0xa4,
  HASH5: 0xa5,
  HASH6: 0xa6,
  HASH7: 0xa7,
  HASH8: 0xa8,
  HASH9: 0xa9,
  HASH10: 0xaa,
  HASH11: 0xab,
  HASH12: 0xac,
  HASH13: 0xad,
  HASH14: 0xae,
  HASH15: 0xaf,
  REMARK: 0xf0,
  REMARK1: 0xf1,
  REMARK2: 0xf2,
  REMARK3: 0xf3,
  REMARK4: 0xf4,
  REMARK5: 0xf5,
  REMARK6: 0xf6,
  REMARK7: 0xf7,
  REMARK8: 0xf8,
  REMARK9: 0xf9,
  REMARK10: 0xfa,
  REMARK11: 0xfb,
  REMARK12: 0xfc,
  REMARK13: 0xfd,
  REMARK14: 0xfe,
  REMARK15: 0xff
};
exports.ATTRIBUTE_USAGE = ATTRIBUTE_USAGE;

var InvalidAttributeUsageError =
/*#__PURE__*/
function (_Error) {
  _inherits(InvalidAttributeUsageError, _Error);

  function InvalidAttributeUsageError(transactionAttributeUsage) {
    var _this;

    _classCallCheck(this, InvalidAttributeUsageError);

    _this = _possibleConstructorReturn(this, (InvalidAttributeUsageError.__proto__ || Object.getPrototypeOf(InvalidAttributeUsageError)).call(this, "Expected transaction attribute usage, " + "found: ".concat(transactionAttributeUsage.toString(16))));
    Object.defineProperty(_this, "transactionAttributeUsage", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    _this.transactionAttributeUsage = transactionAttributeUsage;
    return _this;
  }

  return InvalidAttributeUsageError;
}(Error);

exports.InvalidAttributeUsageError = InvalidAttributeUsageError;

var assertAttributeUsage = function assertAttributeUsage(value) {
  switch (value) {
    case 0x00:
      // ContractHash
      return 0x00;

    case 0x02:
      // ECDH02
      return 0x02;

    case 0x03:
      // ECDH03
      return 0x03;

    case 0x20:
      // Script
      return 0x20;

    case 0x30:
      // Vote
      return 0x30;

    case 0x81:
      // DescriptionUrl
      return 0x81;

    case 0x90:
      // Description
      return 0x90;

    case 0xa1:
      // Hash1
      return 0xa1;

    case 0xa2:
      // Hash2
      return 0xa2;

    case 0xa3:
      // Hash3
      return 0xa3;

    case 0xa4:
      // Hash4
      return 0xa4;

    case 0xa5:
      // Hash5
      return 0xa5;

    case 0xa6:
      // Hash6
      return 0xa6;

    case 0xa7:
      // Hash7
      return 0xa7;

    case 0xa8:
      // Hash8
      return 0xa8;

    case 0xa9:
      // Hash9
      return 0xa9;

    case 0xaa:
      // Hash10
      return 0xaa;

    case 0xab:
      // Hash11
      return 0xab;

    case 0xac:
      // Hash12
      return 0xac;

    case 0xad:
      // Hash13
      return 0xad;

    case 0xae:
      // Hash14
      return 0xae;

    case 0xaf:
      // Hash15
      return 0xaf;

    case 0xf0:
      // Remark
      return 0xf0;

    case 0xf1:
      // Remark1
      return 0xf1;

    case 0xf2:
      // Remark2
      return 0xf2;

    case 0xf3:
      // Remark3
      return 0xf3;

    case 0xf4:
      // Remark4
      return 0xf4;

    case 0xf5:
      // Remark5
      return 0xf5;

    case 0xf6:
      // Remark6
      return 0xf6;

    case 0xf7:
      // Remark7
      return 0xf7;

    case 0xf8:
      // Remark8
      return 0xf8;

    case 0xf9:
      // Remark9
      return 0xf9;

    case 0xfa:
      // Remark10
      return 0xfa;

    case 0xfb:
      // Remark11
      return 0xfb;

    case 0xfc:
      // Remark12
      return 0xfc;

    case 0xfd:
      // Remark13
      return 0xfd;

    case 0xfe:
      // Remark14
      return 0xfe;

    case 0xff:
      // Remark15
      return 0xff;

    default:
      throw new InvalidAttributeUsageError(value);
  }
};

exports.assertAttributeUsage = assertAttributeUsage;

var InvalidJSONAttributeUsageError =
/*#__PURE__*/
function (_Error2) {
  _inherits(InvalidJSONAttributeUsageError, _Error2);

  function InvalidJSONAttributeUsageError(transactionAttributeUsage) {
    var _this2;

    _classCallCheck(this, InvalidJSONAttributeUsageError);

    _this2 = _possibleConstructorReturn(this, (InvalidJSONAttributeUsageError.__proto__ || Object.getPrototypeOf(InvalidJSONAttributeUsageError)).call(this, "Expected transaction attribute usage, " + "found: ".concat(transactionAttributeUsage)));
    Object.defineProperty(_this2, "transactionAttributeUsage", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    _this2.transactionAttributeUsage = transactionAttributeUsage;
    return _this2;
  }

  return InvalidJSONAttributeUsageError;
}(Error);

exports.InvalidJSONAttributeUsageError = InvalidJSONAttributeUsageError;

var toJSONAttributeUsage = function toJSONAttributeUsage(usage) {
  switch (usage) {
    case 0x00:
      return 'ContractHash';

    case 0x02:
      return 'ECDH02';

    case 0x03:
      return 'ECDH03';

    case 0x20:
      return 'Script';

    case 0x30:
      return 'Vote';

    case 0x81:
      return 'DescriptionUrl';

    case 0x90:
      return 'Description';

    case 0xa1:
      return 'Hash1';

    case 0xa2:
      return 'Hash2';

    case 0xa3:
      return 'Hash3';

    case 0xa4:
      return 'Hash4';

    case 0xa5:
      return 'Hash5';

    case 0xa6:
      return 'Hash6';

    case 0xa7:
      return 'Hash7';

    case 0xa8:
      return 'Hash8';

    case 0xa9:
      return 'Hash9';

    case 0xaa:
      return 'Hash10';

    case 0xab:
      return 'Hash11';

    case 0xac:
      return 'Hash12';

    case 0xad:
      return 'Hash13';

    case 0xae:
      return 'Hash14';

    case 0xaf:
      return 'Hash15';

    case 0xf0:
      return 'Remark';

    case 0xf1:
      return 'Remark1';

    case 0xf2:
      return 'Remark2';

    case 0xf3:
      return 'Remark3';

    case 0xf4:
      return 'Remark4';

    case 0xf5:
      return 'Remark5';

    case 0xf6:
      return 'Remark6';

    case 0xf7:
      return 'Remark7';

    case 0xf8:
      return 'Remark8';

    case 0xf9:
      return 'Remark9';

    case 0xfa:
      return 'Remark10';

    case 0xfb:
      return 'Remark11';

    case 0xfc:
      return 'Remark12';

    case 0xfd:
      return 'Remark13';

    case 0xfe:
      return 'Remark14';

    case 0xff:
      return 'Remark15';

    default:
      // eslint-disable-next-line
      usage;
      throw new InvalidAttributeUsageError(usage);
  }
};

exports.toJSONAttributeUsage = toJSONAttributeUsage;