"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _bn = _interopRequireDefault(require("bn.js"));

var _lodash = _interopRequireDefault(require("lodash"));

var _common = _interopRequireDefault(require("../common"));

var _errors = require("../errors");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var BinaryReader =
/*#__PURE__*/
function () {
  function BinaryReader(buffer, index) {
    _classCallCheck(this, BinaryReader);

    Object.defineProperty(this, "buffer", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, "index", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    this.buffer = buffer;
    this.index = index || 0;
  }

  _createClass(BinaryReader, [{
    key: "hasMore",
    value: function hasMore() {
      return this.index < this.buffer.byteLength;
    }
  }, {
    key: "clone",
    value: function clone() {
      return new BinaryReader(this.buffer, this.index);
    }
  }, {
    key: "readBytes",
    value: function readBytes(numBytes) {
      var result = this.buffer.slice(this.index, this.index + numBytes);
      this.index += numBytes;
      return result;
    }
  }, {
    key: "readInt8",
    value: function readInt8() {
      var result = this.buffer.readInt8(this.index);
      this.index += 1;
      return result;
    }
  }, {
    key: "readUInt8",
    value: function readUInt8() {
      var result = this.buffer.readUInt8(this.index);
      this.index += 1;
      return result;
    }
  }, {
    key: "readBoolean",
    value: function readBoolean() {
      return this.readBytes(1)[0] !== 0;
    }
  }, {
    key: "readInt16LE",
    value: function readInt16LE() {
      var result = this.buffer.readInt16LE(this.index);
      this.index += 2;
      return result;
    }
  }, {
    key: "readUInt16LE",
    value: function readUInt16LE() {
      var result = this.buffer.readUInt16LE(this.index);
      this.index += 2;
      return result;
    }
  }, {
    key: "readInt32LE",
    value: function readInt32LE() {
      var result = this.buffer.readInt32LE(this.index);
      this.index += 4;
      return result;
    }
  }, {
    key: "readUInt32LE",
    value: function readUInt32LE() {
      var result = this.buffer.readUInt32LE(this.index);
      this.index += 4;
      return result;
    }
  }, {
    key: "readUInt64LE",
    value: function readUInt64LE() {
      return new _bn.default(this.readBytes(8), 'le');
    }
  }, {
    key: "readInt64LE",
    value: function readInt64LE() {
      var buffer = this.readBytes(8);
      return new _bn.default(buffer, 'le').fromTwos(buffer.length * 8);
    } // NEO specific

  }, {
    key: "readUInt160",
    value: function readUInt160() {
      return _common.default.bufferToUInt160(this.readBytes(_common.default.UINT160_BUFFER_BYTES));
    }
  }, {
    key: "readUInt256",
    value: function readUInt256() {
      return _common.default.bufferToUInt256(this.readBytes(_common.default.UINT256_BUFFER_BYTES));
    }
  }, {
    key: "readFixed8",
    value: function readFixed8() {
      return this.readInt64LE();
    }
  }, {
    key: "readFixedString",
    value: function readFixedString(length) {
      var values = _lodash.default.takeWhile([].concat(_toConsumableArray(this.readBytes(length))), function (value) {
        return value !== 0;
      });

      return Buffer.from(values).toString('utf8');
    }
  }, {
    key: "readArray",
    value: function readArray(read) {
      var max = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0x10000000;
      var count = this.readVarUIntLE(new _bn.default(max)).toNumber();
      var result = [];

      for (var i = 0; i < count; i += 1) {
        result.push(read());
      }

      return result;
    }
  }, {
    key: "readObject",
    value: function readObject(read) {
      var max = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0x10000000;
      var count = this.readVarUIntLE(new _bn.default(max)).toNumber();
      var result = {};

      for (var i = 0; i < count; i += 1) {
        var _read = read(),
            _key = _read.key,
            _value = _read.value;

        result[_key] = _value;
      }

      return result;
    }
  }, {
    key: "readVarBytesLE",
    value: function readVarBytesLE() {
      var max = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : Number.MAX_SAFE_INTEGER;
      return this.readBytes(this.readVarUIntLE(new _bn.default(max)).toNumber());
    }
  }, {
    key: "readVarUIntLE",
    value: function readVarUIntLE() {
      var max = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new _bn.default('18446744073709551615', 10);
      var fb = this.readUInt8();
      var value = new _bn.default(fb);

      if (fb === 0xFD) {
        value = new _bn.default(this.readUInt16LE());
      } else if (fb === 0xFE) {
        value = new _bn.default(this.readUInt32LE());
      } else if (fb === 0xFF) {
        value = this.readUInt64LE();
      }

      if (value.gt(max)) {
        throw new _errors.InvalidFormatError();
      }

      return value;
    }
  }, {
    key: "readVarString",
    value: function readVarString() {
      var max = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0x7fffffc7;
      return this.readVarBytesLE(max).toString('utf8');
    }
  }, {
    key: "readECPoint",
    value: function readECPoint() {
      var firstByte = this.readBytes(1);

      if (firstByte[0] === _common.default.ECPOINT_INFINITY_BYTE) {
        return _common.default.ECPOINT_INFINITY;
      }

      return _common.default.bufferToECPoint(Buffer.concat([firstByte, this.readBytes(_common.default.ECPOINT_BUFFER_BYTES - 1)]));
    }
  }, {
    key: "remaining",
    get: function get() {
      return this.buffer.length - this.index;
    }
  }, {
    key: "remainingBuffer",
    get: function get() {
      return this.buffer.slice(this.index);
    }
  }]);

  return BinaryReader;
}();

exports.default = BinaryReader;