"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _bn = _interopRequireDefault(require("bn.js"));

var _errors = require("../errors");

var _common = _interopRequireDefault(require("../common"));

var _utils = _interopRequireDefault(require("./utils"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _slicedToArray(arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return _sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var BinaryWriter =
/*#__PURE__*/
function () {
  function BinaryWriter() {
    _classCallCheck(this, BinaryWriter);

    Object.defineProperty(this, "buffer", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    this.buffer = Buffer.from([]);
  }

  _createClass(BinaryWriter, [{
    key: "toBuffer",
    value: function toBuffer() {
      return this.buffer;
    }
  }, {
    key: "writeBytes",
    value: function writeBytes(value) {
      this.buffer = Buffer.concat([this.buffer, value]);
      return this;
    }
  }, {
    key: "writeUInt8",
    value: function writeUInt8(value) {
      var buffer = Buffer.allocUnsafe(1);
      buffer.writeUInt8(value, 0);
      return this.writeBytes(buffer);
    }
  }, {
    key: "writeInt16LE",
    value: function writeInt16LE(value) {
      var buffer = Buffer.allocUnsafe(2);
      buffer.writeInt16LE(value, 0);
      return this.writeBytes(buffer);
    }
  }, {
    key: "writeUInt16LE",
    value: function writeUInt16LE(value) {
      var buffer = Buffer.allocUnsafe(2);
      buffer.writeUInt16LE(value, 0);
      return this.writeBytes(buffer);
    }
  }, {
    key: "writeInt32LE",
    value: function writeInt32LE(value) {
      var buffer = Buffer.allocUnsafe(4);
      buffer.writeInt32LE(value, 0);
      return this.writeBytes(buffer);
    }
  }, {
    key: "writeUInt32LE",
    value: function writeUInt32LE(value) {
      var buffer = Buffer.allocUnsafe(4);
      buffer.writeUInt32LE(value, 0);
      return this.writeBytes(buffer);
    }
  }, {
    key: "writeInt64LE",
    value: function writeInt64LE(value) {
      return this.writeBytes(value.toTwos(8 * 8).toArrayLike(Buffer, 'le', 8));
    }
  }, {
    key: "writeUInt64LE",
    value: function writeUInt64LE(value) {
      return this.writeBytes(value.toArrayLike(Buffer, 'le', 8));
    }
  }, {
    key: "writeBoolean",
    value: function writeBoolean(value) {
      this.writeBytes(Buffer.from([value ? 1 : 0]));
      return this;
    } // Special methods that don't fit the LE mold and/or are specific to NEO.

  }, {
    key: "writeUInt160",
    value: function writeUInt160(hash) {
      return this.writeBytes(_common.default.uInt160ToBuffer(hash));
    }
  }, {
    key: "writeUInt256",
    value: function writeUInt256(hash) {
      return this.writeBytes(_common.default.uInt256ToBuffer(hash));
    }
  }, {
    key: "writeFixed8",
    value: function writeFixed8(value) {
      return this.writeInt64LE(value);
    }
  }, {
    key: "writeFixedString",
    value: function writeFixedString(value, length) {
      if (value.length > length) {
        throw new _errors.InvalidFormatError();
      }

      var buffer = Buffer.from(value, 'utf8');

      if (buffer.length > length) {
        throw new _errors.InvalidFormatError();
      }

      this.writeBytes(buffer);

      if (buffer.length < length) {
        this.writeBytes(Buffer.alloc(length - buffer.length, 0));
      }

      return this;
    }
  }, {
    key: "writeArray",
    value: function writeArray(values, write) {
      this.writeVarUIntLE(values.length);
      values.forEach(function (value) {
        return write(value);
      });
      return this;
    }
  }, {
    key: "writeObject",
    value: function writeObject(value, write) {
      var entries = _utils.default.entries(value);

      this.writeVarUIntLE(entries.length);
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = entries[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var _ref3 = _step.value;

          var _ref2 = _slicedToArray(_ref3, 2);

          var _key2 = _ref2[0];
          var _val = _ref2[1];
          write(_key2, _val);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return this;
    }
  }, {
    key: "writeVarBytesLE",
    value: function writeVarBytesLE(value) {
      this.writeVarUIntLE(value.length);
      return this.writeBytes(value);
    }
  }, {
    key: "writeVarUIntLE",
    value: function writeVarUIntLE(valueIn) {
      var value = new _bn.default(valueIn);

      if (value.lt(_utils.default.ZERO)) {
        throw new _errors.InvalidFormatError();
      }

      if (value.lt(_utils.default.FD)) {
        this.writeUInt8(value.toNumber());
      } else if (value.lte(_utils.default.FFFF)) {
        this.writeUInt8(0xFD);
        this.writeUInt16LE(value.toNumber());
      } else if (value.lte(_utils.default.FFFFFFFF)) {
        this.writeUInt8(0xFE);
        this.writeUInt32LE(value.toNumber());
      } else {
        this.writeUInt8(0xFF);
        this.writeUInt64LE(value);
      }

      return this;
    }
  }, {
    key: "writeVarString",
    value: function writeVarString(value, max) {
      var buffer = Buffer.from(value, 'utf8');

      if (max != null) {
        buffer = buffer.slice(0, max);
      }

      return this.writeVarBytesLE(buffer);
    }
  }, {
    key: "writeECPoint",
    value: function writeECPoint(value) {
      if (_common.default.ecPointIsInfinity(value)) {
        return this.writeBytes(Buffer.from([_common.default.ECPOINT_INFINITY_BYTE]));
      }

      return this.writeBytes(_common.default.ecPointToBuffer(value));
    }
  }]);

  return BinaryWriter;
}();

exports.default = BinaryWriter;