"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _bn = _interopRequireDefault(require("bn.js"));

var _errors = require("../errors");

var _common = _interopRequireDefault(require("../common"));

var _utils = _interopRequireDefault(require("./utils"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _slicedToArray(arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return _sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }

var sizeOfUInt8 = 1;
var sizeOfBoolean = 1;
var sizeOfUInt16LE = 2;
var sizeOfUInt32LE = 4;
var sizeOfUInt64LE = 8;
var sizeOfUInt160 = _common.default.UINT160_BUFFER_BYTES;
var sizeOfUInt256 = _common.default.UINT256_BUFFER_BYTES;

var sizeOfECPoint = function sizeOfECPoint(ecPoint) {
  if (_common.default.ecPointIsInfinity(ecPoint)) {
    return 1;
  }

  return _common.default.ECPOINT_BUFFER_BYTES;
};

var sizeOfFixed8 = 8;

var sizeOfVarUIntLE = function sizeOfVarUIntLE(valueIn) {
  var value = new _bn.default(valueIn);

  if (value.lt(_utils.default.ZERO)) {
    throw new _errors.InvalidFormatError();
  }

  if (value.lt(_utils.default.FD)) {
    return sizeOfUInt8;
  } else if (value.lte(_utils.default.FFFF)) {
    return sizeOfUInt8 + sizeOfUInt16LE;
  } else if (value.lte(_utils.default.FFFFFFFF)) {
    return sizeOfUInt8 + sizeOfUInt32LE;
  }

  return sizeOfUInt8 + sizeOfUInt64LE;
};

var sizeOfVarBytesLE = function sizeOfVarBytesLE(value) {
  return sizeOfVarUIntLE(value.length) + value.length;
};

var sizeOfVarString = function sizeOfVarString(value) {
  return sizeOfVarBytesLE(Buffer.from(value, 'utf8'));
};

var sizeOfFixedString = function sizeOfFixedString(length) {
  return length;
};

function sizeOfArray(values, sizeOf) {
  return values.reduce(function (acc, value) {
    return acc + sizeOf(value);
  }, sizeOfVarUIntLE(values.length));
}

function sizeOfObject(obj, sizeOf) {
  var entries = _utils.default.entries(obj);

  return entries.reduce(function (acc, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        value = _ref2[1];

    return acc + sizeOf(key, value);
  }, entries.length);
}

;
var _default = {
  sizeOfUInt8: sizeOfUInt8,
  sizeOfBoolean: sizeOfBoolean,
  sizeOfUInt16LE: sizeOfUInt16LE,
  sizeOfUInt32LE: sizeOfUInt32LE,
  sizeOfUInt64LE: sizeOfUInt64LE,
  sizeOfVarBytesLE: sizeOfVarBytesLE,
  sizeOfVarString: sizeOfVarString,
  sizeOfFixedString: sizeOfFixedString,
  sizeOfArray: sizeOfArray,
  sizeOfObject: sizeOfObject,
  sizeOfUInt160: sizeOfUInt160,
  sizeOfUInt256: sizeOfUInt256,
  sizeOfECPoint: sizeOfECPoint,
  sizeOfFixed8: sizeOfFixed8
};
exports.default = _default;