"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.InvalidParamError = exports.UnknownOpError = void 0;

var _bn = _interopRequireDefault(require("bn.js"));

var _lodash = _interopRequireDefault(require("lodash"));

var _vm = require("../vm");

var _common = _interopRequireDefault(require("../common"));

var _utils = _interopRequireDefault(require("./utils"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var UnknownOpError =
/*#__PURE__*/
function (_Error) {
  _inherits(UnknownOpError, _Error);

  function UnknownOpError(byteCode) {
    var _this;

    _classCallCheck(this, UnknownOpError);

    _this = _possibleConstructorReturn(this, (UnknownOpError.__proto__ || Object.getPrototypeOf(UnknownOpError)).call(this, "Unknown op: ".concat(byteCode)));
    Object.defineProperty(_this, "byteCode", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    _this.byteCode = byteCode;
    return _this;
  }

  return UnknownOpError;
}(Error);

exports.UnknownOpError = UnknownOpError;

var InvalidParamError =
/*#__PURE__*/
function (_Error2) {
  _inherits(InvalidParamError, _Error2);

  function InvalidParamError() {
    _classCallCheck(this, InvalidParamError);

    return _possibleConstructorReturn(this, (InvalidParamError.__proto__ || Object.getPrototypeOf(InvalidParamError)).call(this, 'Invalid Param'));
  }

  return InvalidParamError;
}(Error);

exports.InvalidParamError = InvalidParamError;

var BYTECODE_TO_BYTECODE_BUFFER = _lodash.default.fromPairs(Object.values(_vm.OPCODE_TO_BYTECODE).map(function (byteCode) {
  return [byteCode, Buffer.from([byteCode])];
}));

var ScriptBuilder =
/*#__PURE__*/
function () {
  function ScriptBuilder() {
    _classCallCheck(this, ScriptBuilder);

    Object.defineProperty(this, "buffers", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    this.buffers = [];
  }

  _createClass(ScriptBuilder, [{
    key: "emitPush",
    value: function emitPush(value) {
      if (value.length <= _vm.OPCODE_TO_BYTECODE.PUSHBYTES75) {
        this.emitOpByte(value.length, value);
      } else if (value.length < 0x100) {
        this.emitOp('PUSHDATA1');
        this.emitUInt8(value.length);
        this.emit(value);
      } else if (value.length < 0x10000) {
        this.emitOp('PUSHDATA2');
        this.emitUInt16LE(value.length);
        this.emit(value); // TODO: Check this condition if (data.Length < 0x100000000L)
      } else {
        this.emitOp('PUSHDATA4');
        this.emitUInt32LE(value.length);
        this.emit(value);
      }

      return this;
    }
  }, {
    key: "emitUInt8",
    value: function emitUInt8(value) {
      var buff = Buffer.allocUnsafe(1);
      buff.writeUInt8(value, 0);
      return this.emit(buff);
    }
  }, {
    key: "emitUInt16LE",
    value: function emitUInt16LE(value) {
      var buff = Buffer.allocUnsafe(2);
      buff.writeUInt16LE(value, 0);
      return this.emit(buff);
    }
  }, {
    key: "emitUInt32LE",
    value: function emitUInt32LE(value) {
      var buff = Buffer.allocUnsafe(4);
      buff.writeUInt32LE(value, 0);
      return this.emit(buff);
    }
  }, {
    key: "emitPushInt",
    value: function emitPushInt(valueIn) {
      var value = new _bn.default(valueIn);

      if (value.eq(_utils.default.NEGATIVE_ONE)) {
        return this.emitOp('PUSHM1');
      } else if (value.eq(_utils.default.ZERO)) {
        return this.emitOp('PUSH0');
      } else if (value.gt(_utils.default.ZERO) && value.lt(_utils.default.SIXTEEN)) {
        return this.emitOpByte(_vm.OPCODE_TO_BYTECODE.PUSH1 - 1 + value.toNumber());
      }

      return this.emitPush(_utils.default.toSignedBuffer(value));
    }
  }, {
    key: "emitPushUInt160",
    value: function emitPushUInt160(value) {
      return this.emitPush(_common.default.uInt160ToBuffer(value));
    }
  }, {
    key: "emitPushECPoint",
    value: function emitPushECPoint(ecPoint) {
      return this.emitPush(_common.default.ecPointToBuffer(ecPoint));
    }
  }, {
    key: "emitPushString",
    value: function emitPushString(value) {
      return this.emitPush(Buffer.from(value, 'utf8'));
    }
  }, {
    key: "emitOp",
    value: function emitOp(op, buffer) {
      this.emitOpByte(_vm.OPCODE_TO_BYTECODE[op], buffer);
      return this;
    }
  }, {
    key: "emitPushParam",
    value: function emitPushParam(param) {
      if (_common.default.isUInt160(param)) {
        return this.emitPushUInt160(_common.default.asUInt160(param));
      } else if (typeof param === 'number' || param instanceof _bn.default) {
        return this.emitPushInt(param);
      } else if (typeof param === 'string') {
        return this.emitPushString(param);
      }

      throw new InvalidParamError();
    }
  }, {
    key: "emitAppCall",
    value: function emitAppCall(scriptHash, operation) {
      for (var i = (arguments.length <= 2 ? 0 : arguments.length - 2) - 1; i >= 0; i -= 1) {
        this.emitPushParam(i + 2 < 2 || arguments.length <= i + 2 ? undefined : arguments[i + 2]);
      }

      this.emitPushParam(arguments.length <= 2 ? 0 : arguments.length - 2);
      this.emitOp('PACK');
      this.emitPushParam(operation);
      return this.emitOp('APPCALL', _common.default.uInt160ToBuffer(scriptHash));
    }
  }, {
    key: "emitOpByte",
    value: function emitOpByte(byteCodeIn, buffer) {
      var byteCode = "".concat(byteCodeIn == null ? '' : byteCodeIn);
      var byteCodeBuffer = BYTECODE_TO_BYTECODE_BUFFER[byteCode];

      if (byteCodeBuffer == null) {
        throw new UnknownOpError(byteCode);
      }

      this.emit(byteCodeBuffer);
      this.emit(buffer);
      return this;
    }
  }, {
    key: "emit",
    value: function emit(buffer) {
      if (buffer != null) {
        this.buffers.push(buffer);
      }

      return this;
    }
  }, {
    key: "build",
    value: function build() {
      return Buffer.concat(this.buffers);
    }
  }]);

  return ScriptBuilder;
}();

exports.default = ScriptBuilder;