"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _bn = _interopRequireDefault(require("bn.js"));

var _lodash = _interopRequireDefault(require("lodash"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _slicedToArray(arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return _sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

var USHORT_MAX_NUMBER = 65535;
var USHORT_MAX_NUMBER_PLUS_ONE = 65535 + 1;
var USHORT_MAX = new _bn.default(USHORT_MAX_NUMBER);
var USHORT_MAX_PLUS_ONE = USHORT_MAX.addn(1);
var UINT_MAX_NUMBER = 4294967295;
var UINT_MAX = new _bn.default(UINT_MAX_NUMBER);
var ZERO = new _bn.default(0);
var ONE = new _bn.default(1);
var NEGATIVE_ONE = new _bn.default(-1);
var ONE_HUNDRED_MILLION = new _bn.default(100000000);

var toSignedBuffer = function toSignedBuffer(value) {
  var buff = value.toArrayLike(Buffer, 'le');

  if (value.isNeg()) {
    return buff;
  }

  return value.toArrayLike(Buffer, 'le', buff.length + 1);
};

var fromSignedBuffer = function fromSignedBuffer(value) {
  return value.length === 0 ? ZERO : new _bn.default(value, 'le').fromTwos(value.length * 8);
}; // TODO: This might be incorrect with the shenanigans we're doing above


var not = function not(value) {
  return value.notn(value.bitLength());
};

var getBoolean = function getBoolean(value) {
  return value.some(function (byte) {
    return byte !== 0;
  });
};

var booleanToBuffer = function booleanToBuffer(value) {
  return Buffer.from([value ? 1 : 0]);
};

var toASCII = function toASCII(bytes) {
  var result = '';

  for (var i = 0; i < bytes.length; i += 1) {
    result += String.fromCharCode(bytes.readUInt8(i));
  }

  return result;
};

var toUTF8 = function toUTF8(bytes) {
  return bytes.toString('utf8');
};

function keys(obj) {
  return Object.keys(obj);
}

function values(obj) {
  return Object.values(obj);
}

function entries(obj) {
  return Object.entries(obj);
}

var reverse = function reverse(src) {
  var out = Buffer.allocUnsafe(src.length);

  for (var i = 0, j = src.length - 1; i <= j; i += 1, j -= 1) {
    out[i] = src[j];
    out[j] = src[i];
  }

  return out;
};

var calculateClaimAmount = function () {
  var _ref2 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2(_ref) {
    var coins, decrementInterval, generationAmount, getSystemFee, grouped, claimed;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            coins = _ref.coins, decrementInterval = _ref.decrementInterval, generationAmount = _ref.generationAmount, getSystemFee = _ref.getSystemFee;
            grouped = values(_lodash.default.groupBy(coins, function (coin) {
              return "".concat(coin.startHeight, ":").concat(coin.endHeight);
            }));
            _context2.next = 4;
            return Promise.all(grouped.map(function () {
              var _ref3 = _asyncToGenerator(
              /*#__PURE__*/
              regeneratorRuntime.mark(function _callee(coinsGroup) {
                var _coinsGroup$, startHeight, endHeight, amount, ustart, istart, uend, iend, _ref4, _ref5, sysFeeEnd, sysFeeStart, totalValue;

                return regeneratorRuntime.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        _coinsGroup$ = coinsGroup[0], startHeight = _coinsGroup$.startHeight, endHeight = _coinsGroup$.endHeight;
                        amount = ZERO;
                        ustart = Math.floor(startHeight / decrementInterval);

                        if (ustart < generationAmount.length) {
                          istart = startHeight % decrementInterval;
                          uend = Math.floor(endHeight / decrementInterval);
                          iend = endHeight % decrementInterval;

                          if (uend >= generationAmount.length) {
                            uend = generationAmount.length;
                            iend = 0;
                          }

                          if (iend === 0) {
                            uend -= 1;
                            iend = decrementInterval;
                          }

                          while (ustart < uend) {
                            amount = amount.addn((decrementInterval - istart) * generationAmount[ustart]);
                            ustart += 1;
                            istart = 0;
                          }

                          amount = amount.addn((iend - istart) * generationAmount[ustart]);
                        }

                        _context.next = 6;
                        return Promise.all([getSystemFee(endHeight - 1), startHeight === 0 ? Promise.resolve(ZERO) : getSystemFee(startHeight - 1)]);

                      case 6:
                        _ref4 = _context.sent;
                        _ref5 = _slicedToArray(_ref4, 2);
                        sysFeeEnd = _ref5[0];
                        sysFeeStart = _ref5[1];
                        amount = amount.add(sysFeeEnd.sub(sysFeeStart).div(ONE_HUNDRED_MILLION));
                        totalValue = coinsGroup.reduce(function (acc, _ref6) {
                          var value = _ref6.value;
                          return acc.add(value);
                        }, ZERO);
                        return _context.abrupt("return", [totalValue, amount]);

                      case 13:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee, this);
              }));

              return function (_x2) {
                return _ref3.apply(this, arguments);
              };
            }()));

          case 4:
            claimed = _context2.sent;
            return _context2.abrupt("return", claimed.reduce(function (acc, _ref7) {
              var _ref8 = _slicedToArray(_ref7, 2),
                  value = _ref8[0],
                  amount = _ref8[1];

              return acc.add(value.div(ONE_HUNDRED_MILLION).mul(amount));
            }, ZERO));

          case 6:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function calculateClaimAmount(_x) {
    return _ref2.apply(this, arguments);
  };
}();

var randomUInt = function randomUInt() {
  return Math.floor(Math.random() * UINT_MAX_NUMBER);
};

var _default = {
  FD: new _bn.default(0xFD),
  FFFF: new _bn.default(0xFFFF),
  FFFFFFFF: new _bn.default(0xFFFFFFFF),
  ZERO: ZERO,
  ONE: ONE,
  NEGATIVE_ONE: NEGATIVE_ONE,
  INT_MAX_VALUE: new _bn.default(2147483647),
  SATOSHI: ONE,
  NEGATIVE_SATOSHI: NEGATIVE_ONE,
  USHORT_MAX_NUMBER: USHORT_MAX_NUMBER,
  USHORT_MAX_NUMBER_PLUS_ONE: USHORT_MAX_NUMBER_PLUS_ONE,
  USHORT_MAX: USHORT_MAX,
  USHORT_MAX_PLUS_ONE: USHORT_MAX_PLUS_ONE,
  UINT_MAX_NUMBER: UINT_MAX_NUMBER,
  UINT_MAX: UINT_MAX,
  ONE_HUNDRED_MILLION: ONE_HUNDRED_MILLION,
  EIGHT: new _bn.default(8),
  TEN: new _bn.default(10),
  SIXTEEN: new _bn.default(16),
  toSignedBuffer: toSignedBuffer,
  fromSignedBuffer: fromSignedBuffer,
  not: not,
  getBoolean: getBoolean,
  booleanToBuffer: booleanToBuffer,
  toASCII: toASCII,
  toUTF8: toUTF8,
  keys: keys,
  values: values,
  entries: entries,
  reverse: reverse,
  calculateClaimAmount: calculateClaimAmount,
  randomUInt: randomUInt
};
exports.default = _default;