"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.assertSysCallName = exports.InvalidSysCallNameError = exports.SYS_CALL_NAME = exports.OPCODE_TO_BYTECODE = exports.BYTECODE_TO_OPCODE = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var OPCODE_PAIRS = [[0x00, 'PUSH0'], [0x01, 'PUSHBYTES1']].concat(_lodash.default.range(2, 74).map(function (idx) {
  return [idx, "PUSHBYTES".concat(idx)];
})).concat([[0x4B, 'PUSHBYTES75'], [0x4C, 'PUSHDATA1'], [0x4D, 'PUSHDATA2'], [0x4E, 'PUSHDATA4'], [0x4F, 'PUSHM1'], [0x51, 'PUSH1'], [0x52, 'PUSH2'], [0x53, 'PUSH3'], [0x54, 'PUSH4'], [0x55, 'PUSH5'], [0x56, 'PUSH6'], [0x57, 'PUSH7'], [0x58, 'PUSH8'], [0x59, 'PUSH9'], [0x5A, 'PUSH10'], [0x5B, 'PUSH11'], [0x5C, 'PUSH12'], [0x5D, 'PUSH13'], [0x5E, 'PUSH14'], [0x5F, 'PUSH15'], [0x60, 'PUSH16'], [0x61, 'NOP'], [0x62, 'JMP'], [0x63, 'JMPIF'], [0x64, 'JMPIFNOT'], [0x65, 'CALL'], [0x66, 'RET'], [0x67, 'APPCALL'], [0x68, 'SYSCALL'], [0x69, 'TAILCALL'], [0x6A, 'DUPFROMALTSTACK'], [0x6B, 'TOALTSTACK'], [0x6C, 'FROMALTSTACK'], [0x6D, 'XDROP'], [0x72, 'XSWAP'], [0x73, 'XTUCK'], [0x74, 'DEPTH'], [0x75, 'DROP'], [0x76, 'DUP'], [0x77, 'NIP'], [0x78, 'OVER'], [0x79, 'PICK'], [0x7A, 'ROLL'], [0x7B, 'ROT'], [0x7C, 'SWAP'], [0x7D, 'TUCK'], [0x7E, 'CAT'], [0x7F, 'SUBSTR'], [0x80, 'LEFT'], [0x81, 'RIGHT'], [0x82, 'SIZE'], [0x83, 'INVERT'], [0x84, 'AND'], [0x85, 'OR'], [0x86, 'XOR'], [0x87, 'EQUAL'], [0x88, 'OP_EQUALVERIFY'], [0x89, 'OP_RESERVED1'], [0x8A, 'OP_RESERVED2'], [0x8B, 'INC'], [0x8C, 'DEC'], [0x8D, 'SIGN'], [0x8F, 'NEGATE'], [0x90, 'ABS'], [0x91, 'NOT'], [0x92, 'NZ'], [0x93, 'ADD'], [0x94, 'SUB'], [0x95, 'MUL'], [0x96, 'DIV'], [0x97, 'MOD'], [0x98, 'SHL'], [0x99, 'SHR'], [0x9A, 'BOOLAND'], [0x9B, 'BOOLOR'], [0x9C, 'NUMEQUAL'], [0x9E, 'NUMNOTEQUAL'], [0x9F, 'LT'], [0xA0, 'GT'], [0xA1, 'LTE'], [0xA2, 'GTE'], [0xA3, 'MIN'], [0xA4, 'MAX'], [0xA5, 'WITHIN'], [0xA7, 'SHA1'], [0xA8, 'SHA256'], [0xA9, 'HASH160'], [0xAA, 'HASH256'], [0xAC, 'CHECKSIG'], [0xAE, 'CHECKMULTISIG'], [0xC0, 'ARRAYSIZE'], [0xC1, 'PACK'], [0xC2, 'UNPACK'], [0xC3, 'PICKITEM'], [0xC4, 'SETITEM'], [0xC5, 'NEWARRAY'], [0xC6, 'NEWSTRUCT'], [0xF0, 'THROW'], [0xF1, 'THROWIFNOT']]);

var BYTECODE_TO_OPCODE = _lodash.default.fromPairs(OPCODE_PAIRS);

exports.BYTECODE_TO_OPCODE = BYTECODE_TO_OPCODE;

var OPCODE_TO_BYTECODE = _lodash.default.mapValues(_lodash.default.invert(BYTECODE_TO_OPCODE), function (val) {
  return parseInt(val, 10);
});

exports.OPCODE_TO_BYTECODE = OPCODE_TO_BYTECODE;
var SYS_CALL_NAME = {
  RUNTIME_GET_TRIGGER: 'Neo.Runtime.GetTrigger',
  RUNTIME_CHECK_WITNESS: 'Neo.Runtime.CheckWitness',
  RUNTIME_NOTIFY: 'Neo.Runtime.Notify',
  RUNTIME_LOG: 'Neo.Runtime.Log',
  BLOCKCHAIN_GET_HEIGHT: 'Neo.Blockchain.GetHeight',
  BLOCKCHAIN_GET_HEADER: 'Neo.Blockchain.GetHeader',
  BLOCKCHAIN_GET_BLOCK: 'Neo.Blockchain.GetBlock',
  BLOCKCHAIN_GET_TRANSACTION: 'Neo.Blockchain.GetTransaction',
  BLOCKCHAIN_GET_ACCOUNT: 'Neo.Blockchain.GetAccount',
  BLOCKCHAIN_GET_VALIDATORS: 'Neo.Blockchain.GetValidators',
  BLOCKCHAIN_GET_ASSET: 'Neo.Blockchain.GetAsset',
  BLOCKCHAIN_GET_CONTRACT: 'Neo.Blockchain.GetContract',
  HEADER_GET_HASH: 'Neo.Header.GetHash',
  HEADER_GET_VERSION: 'Neo.Header.GetVersion',
  HEADER_GET_PREVHASH: 'Neo.Header.GetPrevHash',
  HEADER_GET_MERKLEROOT: 'Neo.Header.GetMerkleRoot',
  HEADER_GET_TIMESTAMP: 'Neo.Header.GetTimestamp',
  HEADER_GET_CONSENSUSDATA: 'Neo.Header.GetConsensusData',
  HEADER_GET_NEXTCONSENSUS: 'Neo.Header.GetNextConsensus',
  BLOCK_GET_TRANSACTIONCOUNT: 'Neo.Block.GetTransactionCount',
  BLOCK_GET_TRANSACTIONS: 'Neo.Block.GetTransactions',
  BLOCK_GET_TRANSACTION: 'Neo.Block.GetTransaction',
  TRANSACTION_GET_HASH: 'Neo.Transaction.GetHash',
  TRANSACTION_GET_TYPE: 'Neo.Transaction.GetType',
  TRANSACTION_GET_ATTRIBUTES: 'Neo.Transaction.GetAttributes',
  TRANSACTION_GET_INPUTS: 'Neo.Transaction.GetInputs',
  TRANSACTION_GET_OUTPUTS: 'Neo.Transaction.GetOutputs',
  TRANSACTION_GET_REFERENCES: 'Neo.Transaction.GetReferences',
  ATTRIBUTE_GET_USAGE: 'Neo.Attribute.GetUsage',
  ATTRIBUTE_GET_DATA: 'Neo.Attribute.GetData',
  INPUT_GET_HASH: 'Neo.Input.GetHash',
  INPUT_GET_INDEX: 'Neo.Input.GetIndex',
  OUTPUT_GET_ASSETID: 'Neo.Output.GetAssetId',
  OUTPUT_GET_VALUE: 'Neo.Output.GetValue',
  OUTPUT_GET_SCRIPTHASH: 'Neo.Output.GetScriptHash',
  ACCOUNT_GET_SCRIPTHASH: 'Neo.Account.GetScriptHash',
  ACCOUNT_GET_VOTES: 'Neo.Account.GetVotes',
  ACCOUNT_GET_BALANCE: 'Neo.Account.GetBalance',
  ASSET_GET_ASSETID: 'Neo.Asset.GetAssetId',
  ASSET_GET_ASSETTYPE: 'Neo.Asset.GetAssetType',
  ASSET_GET_AMOUNT: 'Neo.Asset.GetAmount',
  ASSET_GET_AVAILABLE: 'Neo.Asset.GetAvailable',
  ASSET_GET_PRECISION: 'Neo.Asset.GetPrecision',
  ASSET_GET_OWNER: 'Neo.Asset.GetOwner',
  ASSET_GET_ADMIN: 'Neo.Asset.GetAdmin',
  ASSET_GET_ISSUER: 'Neo.Asset.GetIssuer',
  CONTRACT_GET_SCRIPT: 'Neo.Contract.GetScript',
  STORAGE_GET_CONTEXT: 'Neo.Storage.GetContext',
  STORAGE_GET: 'Neo.Storage.Get',
  ACCOUNT_SET_VOTES: 'Neo.Account.SetVotes',
  VALIDATOR_REGISTER: 'Neo.Validator.Register',
  ASSET_CREATE: 'Neo.Asset.Create',
  ASSET_RENEW: 'Neo.Asset.Renew',
  CONTRACT_CREATE: 'Neo.Contract.Create',
  CONTRACT_MIGRATE_: 'Neo.Contract.Migrate',
  CONTRACT_GET_STORAGE_CONTEXT: 'Neo.Contract.GetStorageContext',
  CONTRACT_DESTROY: 'Neo.Contract.Destroy',
  STORAGE_PUT: 'Neo.Storage.Put',
  STORAGE_DELETE: 'Neo.Storage.Delete',
  EXECUTION_ENGINE_GET_SCRIPT_CONTAINER: 'System.ExecutionEngine.GetScriptContainer',
  EXECUTION_ENGINE_GET_EXECUTING_SCRIPT_HASH: 'System.ExecutionEngine.GetExecutingScriptHash',
  EXECUTION_ENGINE_GET_CALLING_SCRIPT_HASH: 'System.ExecutionEngine.GetCallingScriptHash',
  EXECUTION_ENGINE_GET_ENTRY_SCRIPT_HASH: 'System.ExecutionEngine.GetEntryScriptHash'
};
exports.SYS_CALL_NAME = SYS_CALL_NAME;

var InvalidSysCallNameError =
/*#__PURE__*/
function (_Error) {
  _inherits(InvalidSysCallNameError, _Error);

  function InvalidSysCallNameError(value) {
    var _this;

    _classCallCheck(this, InvalidSysCallNameError);

    _this = _possibleConstructorReturn(this, (InvalidSysCallNameError.__proto__ || Object.getPrototypeOf(InvalidSysCallNameError)).call(this, "Expected sys call name, found: ".concat(value)));
    Object.defineProperty(_this, "value", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: void 0
    });
    _this.value = value;
    return _this;
  }

  return InvalidSysCallNameError;
}(Error);

exports.InvalidSysCallNameError = InvalidSysCallNameError;

var assertSysCallName = function assertSysCallName(value) {
  switch (value) {
    case SYS_CALL_NAME.RUNTIME_GET_TRIGGER:
      return SYS_CALL_NAME.RUNTIME_GET_TRIGGER;

    case SYS_CALL_NAME.RUNTIME_CHECK_WITNESS:
      return SYS_CALL_NAME.RUNTIME_CHECK_WITNESS;

    case SYS_CALL_NAME.RUNTIME_NOTIFY:
      return SYS_CALL_NAME.RUNTIME_NOTIFY;

    case SYS_CALL_NAME.RUNTIME_LOG:
      return SYS_CALL_NAME.RUNTIME_LOG;

    case SYS_CALL_NAME.BLOCKCHAIN_GET_HEIGHT:
      return SYS_CALL_NAME.BLOCKCHAIN_GET_HEIGHT;

    case SYS_CALL_NAME.BLOCKCHAIN_GET_HEADER:
      return SYS_CALL_NAME.BLOCKCHAIN_GET_HEADER;

    case SYS_CALL_NAME.BLOCKCHAIN_GET_BLOCK:
      return SYS_CALL_NAME.BLOCKCHAIN_GET_BLOCK;

    case SYS_CALL_NAME.BLOCKCHAIN_GET_TRANSACTION:
      return SYS_CALL_NAME.BLOCKCHAIN_GET_TRANSACTION;

    case SYS_CALL_NAME.BLOCKCHAIN_GET_ACCOUNT:
      return SYS_CALL_NAME.BLOCKCHAIN_GET_ACCOUNT;

    case SYS_CALL_NAME.BLOCKCHAIN_GET_VALIDATORS:
      return SYS_CALL_NAME.BLOCKCHAIN_GET_VALIDATORS;

    case SYS_CALL_NAME.BLOCKCHAIN_GET_ASSET:
      return SYS_CALL_NAME.BLOCKCHAIN_GET_ASSET;

    case SYS_CALL_NAME.BLOCKCHAIN_GET_CONTRACT:
      return SYS_CALL_NAME.BLOCKCHAIN_GET_CONTRACT;

    case SYS_CALL_NAME.HEADER_GET_HASH:
      return SYS_CALL_NAME.HEADER_GET_HASH;

    case SYS_CALL_NAME.HEADER_GET_VERSION:
      return SYS_CALL_NAME.HEADER_GET_VERSION;

    case SYS_CALL_NAME.HEADER_GET_PREVHASH:
      return SYS_CALL_NAME.HEADER_GET_PREVHASH;

    case SYS_CALL_NAME.HEADER_GET_MERKLEROOT:
      return SYS_CALL_NAME.HEADER_GET_MERKLEROOT;

    case SYS_CALL_NAME.HEADER_GET_TIMESTAMP:
      return SYS_CALL_NAME.HEADER_GET_TIMESTAMP;

    case SYS_CALL_NAME.HEADER_GET_CONSENSUSDATA:
      return SYS_CALL_NAME.HEADER_GET_CONSENSUSDATA;

    case SYS_CALL_NAME.HEADER_GET_NEXTCONSENSUS:
      return SYS_CALL_NAME.HEADER_GET_NEXTCONSENSUS;

    case SYS_CALL_NAME.BLOCK_GET_TRANSACTIONCOUNT:
      return SYS_CALL_NAME.BLOCK_GET_TRANSACTIONCOUNT;

    case SYS_CALL_NAME.BLOCK_GET_TRANSACTIONS:
      return SYS_CALL_NAME.BLOCK_GET_TRANSACTIONS;

    case SYS_CALL_NAME.BLOCK_GET_TRANSACTION:
      return SYS_CALL_NAME.BLOCK_GET_TRANSACTION;

    case SYS_CALL_NAME.TRANSACTION_GET_HASH:
      return SYS_CALL_NAME.TRANSACTION_GET_HASH;

    case SYS_CALL_NAME.TRANSACTION_GET_TYPE:
      return SYS_CALL_NAME.TRANSACTION_GET_TYPE;

    case SYS_CALL_NAME.TRANSACTION_GET_ATTRIBUTES:
      return SYS_CALL_NAME.TRANSACTION_GET_ATTRIBUTES;

    case SYS_CALL_NAME.TRANSACTION_GET_INPUTS:
      return SYS_CALL_NAME.TRANSACTION_GET_INPUTS;

    case SYS_CALL_NAME.TRANSACTION_GET_OUTPUTS:
      return SYS_CALL_NAME.TRANSACTION_GET_OUTPUTS;

    case SYS_CALL_NAME.TRANSACTION_GET_REFERENCES:
      return SYS_CALL_NAME.TRANSACTION_GET_REFERENCES;

    case SYS_CALL_NAME.ATTRIBUTE_GET_USAGE:
      return SYS_CALL_NAME.ATTRIBUTE_GET_USAGE;

    case SYS_CALL_NAME.ATTRIBUTE_GET_DATA:
      return SYS_CALL_NAME.ATTRIBUTE_GET_DATA;

    case SYS_CALL_NAME.INPUT_GET_HASH:
      return SYS_CALL_NAME.INPUT_GET_HASH;

    case SYS_CALL_NAME.INPUT_GET_INDEX:
      return SYS_CALL_NAME.INPUT_GET_INDEX;

    case SYS_CALL_NAME.OUTPUT_GET_ASSETID:
      return SYS_CALL_NAME.OUTPUT_GET_ASSETID;

    case SYS_CALL_NAME.OUTPUT_GET_VALUE:
      return SYS_CALL_NAME.OUTPUT_GET_VALUE;

    case SYS_CALL_NAME.OUTPUT_GET_SCRIPTHASH:
      return SYS_CALL_NAME.OUTPUT_GET_SCRIPTHASH;

    case SYS_CALL_NAME.ACCOUNT_GET_SCRIPTHASH:
      return SYS_CALL_NAME.ACCOUNT_GET_SCRIPTHASH;

    case SYS_CALL_NAME.ACCOUNT_GET_VOTES:
      return SYS_CALL_NAME.ACCOUNT_GET_VOTES;

    case SYS_CALL_NAME.ACCOUNT_GET_BALANCE:
      return SYS_CALL_NAME.ACCOUNT_GET_BALANCE;

    case SYS_CALL_NAME.ASSET_GET_ASSETID:
      return SYS_CALL_NAME.ASSET_GET_ASSETID;

    case SYS_CALL_NAME.ASSET_GET_ASSETTYPE:
      return SYS_CALL_NAME.ASSET_GET_ASSETTYPE;

    case SYS_CALL_NAME.ASSET_GET_AMOUNT:
      return SYS_CALL_NAME.ASSET_GET_AMOUNT;

    case SYS_CALL_NAME.ASSET_GET_AVAILABLE:
      return SYS_CALL_NAME.ASSET_GET_AVAILABLE;

    case SYS_CALL_NAME.ASSET_GET_PRECISION:
      return SYS_CALL_NAME.ASSET_GET_PRECISION;

    case SYS_CALL_NAME.ASSET_GET_OWNER:
      return SYS_CALL_NAME.ASSET_GET_OWNER;

    case SYS_CALL_NAME.ASSET_GET_ADMIN:
      return SYS_CALL_NAME.ASSET_GET_ADMIN;

    case SYS_CALL_NAME.ASSET_GET_ISSUER:
      return SYS_CALL_NAME.ASSET_GET_ISSUER;

    case SYS_CALL_NAME.CONTRACT_GET_SCRIPT:
      return SYS_CALL_NAME.CONTRACT_GET_SCRIPT;

    case SYS_CALL_NAME.STORAGE_GET_CONTEXT:
      return SYS_CALL_NAME.STORAGE_GET_CONTEXT;

    case SYS_CALL_NAME.STORAGE_GET:
      return SYS_CALL_NAME.STORAGE_GET;

    case SYS_CALL_NAME.ACCOUNT_SET_VOTES:
      return SYS_CALL_NAME.ACCOUNT_SET_VOTES;

    case SYS_CALL_NAME.VALIDATOR_REGISTER:
      return SYS_CALL_NAME.VALIDATOR_REGISTER;

    case SYS_CALL_NAME.ASSET_CREATE:
      return SYS_CALL_NAME.ASSET_CREATE;

    case SYS_CALL_NAME.ASSET_RENEW:
      return SYS_CALL_NAME.ASSET_RENEW;

    case SYS_CALL_NAME.CONTRACT_CREATE:
      return SYS_CALL_NAME.CONTRACT_CREATE;

    case SYS_CALL_NAME.CONTRACT_MIGRATE_:
      return SYS_CALL_NAME.CONTRACT_MIGRATE_;

    case SYS_CALL_NAME.CONTRACT_GET_STORAGE_CONTEXT:
      return SYS_CALL_NAME.CONTRACT_GET_STORAGE_CONTEXT;

    case SYS_CALL_NAME.CONTRACT_DESTROY:
      return SYS_CALL_NAME.CONTRACT_DESTROY;

    case SYS_CALL_NAME.STORAGE_PUT:
      return SYS_CALL_NAME.STORAGE_PUT;

    case SYS_CALL_NAME.STORAGE_DELETE:
      return SYS_CALL_NAME.STORAGE_DELETE;

    case SYS_CALL_NAME.EXECUTION_ENGINE_GET_SCRIPT_CONTAINER:
      return SYS_CALL_NAME.EXECUTION_ENGINE_GET_SCRIPT_CONTAINER;

    case SYS_CALL_NAME.EXECUTION_ENGINE_GET_EXECUTING_SCRIPT_HASH:
      return SYS_CALL_NAME.EXECUTION_ENGINE_GET_EXECUTING_SCRIPT_HASH;

    case SYS_CALL_NAME.EXECUTION_ENGINE_GET_CALLING_SCRIPT_HASH:
      return SYS_CALL_NAME.EXECUTION_ENGINE_GET_CALLING_SCRIPT_HASH;

    case SYS_CALL_NAME.EXECUTION_ENGINE_GET_ENTRY_SCRIPT_HASH:
      return SYS_CALL_NAME.EXECUTION_ENGINE_GET_ENTRY_SCRIPT_HASH;

    default:
      throw new InvalidSysCallNameError(value);
  }
};

exports.assertSysCallName = assertSysCallName;